/********************************************************************
* NAME:                                                             *
*                                                                   *
* PURPOSE:                                                          *
*                                                                   *
* GLOBAL VARIABLES:                                                 *
*                                                                   *
* Variable:                                                         *
*                                                                   *
* Type:                                                             *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* DEVELOPMENT HISTORY:                                              *
*                                                                   *
* Date:                                                             *
*                                                                   *
* Author:                                                           *
*                                                                   *
* Change Id:                                                        *
*                                                                   *
* Release:                                                          *
*                                                                   *
* Description of change:                                            *
*                                                                   *
*********************************************************************/

#ifndef __cmd_loc_h__
#define __cmd_loc_h__

#define FLASH_LOG_FILE "log.txt"

#define FLASH_LOG_EXTENSION "log"

typedef enum {
              FC_NONE = -1,
              FC_SET_DEBUG_LEVEL = 1,
              FC_SET_REDIRECTION,
              FC_NO_ERASE,
              FC_READ,
              FC_RAW_READ,
              FC_READ_ALL,
              FC_RAW_READ_ALL,
              FC_READ_MCU,
              FC_RAW_READ_MCU,
              FC_READ_PPM,
              FC_RAW_READ_PPM,
              FC_READ_PMM,
              FC_RAW_READ_PMM,
              FC_READ_EEPROM,
              FC_RAW_READ_EEPROM,
              FC_WRITE,
              FC_RAW_WRITE,
              FC_WRITE_PHONEINFO_AND_CHECKSUM,
              FC_WRITE_PHONEINFO,
              FC_WRITE_SECURITY_PHONEINFO,
              FC_LPT,
              FC_COM,
              FC_ERASE,
              FC_ERASEALL,
              FC_CONFIRM,
              FC_READ_FULLFLASH,
              FC_RAW_READ_FULLFLASH,
              FC_EMULATE,
              FC_RAW_EMULATE,
              FC_CHECK_CHECKSUMS,   
              FC_FIX_CHECKSUMS,
              FC_CHECK_IMEI,
              FC_FIX_IMEI,
              FC_CHECK_SECCODE,
              FC_FIX_SECCODE,
              FC_CLEAR_SECCODE_COUNTER,
              FC_SAFEFLASHING,
              FC_RESETSETTINGS,
              FC_RESETPHONE,
              FC_SETSIMLOCK,
              FC_CONVERTTOKNK,
              FC_SETCONTRAST,
              FC_SET_FAID,
              FC_CONVERTWTTOFLS,
              FC_CONVERTWTTOKNK,
              FC_RESETSPLOCK,
              FC_SETIMEI,
	      FC_UPLOADCODE,
             } eCMDLINE_COMMAND;

/* always the last command */
#define CMDLINE_MAX_COMMANDS FC_UPLOADCODE

typedef enum {
              FCAT_NONE = 0,
              FCAT_NUMBER,
              FCAT_STRING
             } eCMDLINE_COMMAND_ARGUMENT_TYPE;

#define CMDLINE_MAX_LONGCMD_LEN 32
#define CMDLINE_MAX_SHORTCMD_LEN 8
#define CMDLINE_MAX_CMD_ARGS 4
#define CMDLINE_MAX_HELP_LEN 48

typedef struct {
                char longcmd_str[CMDLINE_MAX_LONGCMD_LEN];
                char shortcmd_str[CMDLINE_MAX_SHORTCMD_LEN];
                eCMDLINE_COMMAND cmd;
                eCMDLINE_COMMAND_ARGUMENT_TYPE argtype[CMDLINE_MAX_CMD_ARGS];
                UINT8 U8num_of_args;
                BOOL Boptionalargs;
                BOOL Bneedsanalyze; /* whether the command needs the phone info struct */
                BOOL Bneedflasher;
                BOOL Bneedinfo; /* execute FLS1GetPhoneInfo at all */
                BOOL Bneedfchk; /* needs to calc FCHK or not - mostly for phone info */
                BOOL Bneedmbus; /* whether the command uses MBUS communication */
                char help[CMDLINE_MAX_HELP_LEN];
               } tCMDLINE_CMDBLOCK;

static const UINT8 U8num_of_knokcmds = CMDLINE_MAX_COMMANDS;
static const tCMDLINE_CMDBLOCK knokcmd[] =
               {
                 {
                  "setdebuglevel",
                  "sd",
                  FC_SET_DEBUG_LEVEL,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,                        
                  FALSE,
                  FALSE,
                  FALSE,
                  "sets the debug level"
                 },
                 {
                  "setredirection",
                  "sr",
                  FC_SET_REDIRECTION,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "sets redirection"
                 },
                 {
                  "noerase",
                  "ne",
                  FC_NO_ERASE,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "disables erase before write to flash"
                 },
                 {
                  "read",
                  "r",
                  FC_READ,
                  {FCAT_NUMBER,FCAT_NUMBER},
                  2,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads flash into KNK format"
                 },
                 {
                  "rawread",
                  "rr",
                  FC_RAW_READ,
                  {FCAT_NUMBER,FCAT_NUMBER},
                  2,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads flash into FLS format"
                 },
                 {
                  "readall",
                  "ra",
                  FC_READ_ALL,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads all parts of phone into KNK files"
                 },
                 {
                  "rawreadall",
                  "rra",
                  FC_RAW_READ_ALL,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads all parts of phone into FLS files"
                 },
                 {
                  "readmcu",
                  "rm",
                  FC_READ_MCU,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads MCU blocks into KNK files"
                 },
                 {
                  "rawreadmcu",
                  "rrm",
                  FC_RAW_READ_MCU,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads MCU blocks into FLS files"
                 },
                 {
                  "readppm",
                  "rppm",
                  FC_READ_PPM,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads PPM blocks into KNK files"
                 },
                 {
                  "rawreadppm",
                  "rrppm",
                  FC_RAW_READ_PPM,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads PPM blocks into FLS files"
                 },
                 {
                  "readpmm",
                  "rpmm",
                  FC_READ_PMM,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads PMM blocks into KNK files"
                 },
                 {
                  "rawreadpmm",
                  "rrpmm",
                  FC_RAW_READ_PMM,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads PMM blocks into FLS files"
                 },
                 {
                  "readeeprom",
                  "re",
                  FC_READ_EEPROM,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads EEPROM into KNK files"
                 },
                 {
                  "rawreadeeprom",
                  "rre",
                  FC_RAW_READ_EEPROM,
                  {FCAT_NUMBER},
                  1,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads EEPROM into FLS files"
                 },
                 {
                  "write",
                  "w",
                  FC_WRITE,
                  {FCAT_STRING,FCAT_NUMBER,FCAT_NUMBER},
                  3,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "writes to phone from KNK file"
                 },
                 {
                  "rawwrite",
                  "rw",
                  FC_RAW_WRITE,
                  {FCAT_STRING,FCAT_NUMBER,FCAT_NUMBER,FCAT_NUMBER},
                  4,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "writes to phone from FLS file"
                 },
                 {
                  "writephoneinfoandchecksum",
                  "wpc",
                  FC_WRITE_PHONEINFO_AND_CHECKSUM,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  "logs phone information and PPM checksum to file"
                 },
                 {
                  "writephoneinfo",
                  "wp",
                  FC_WRITE_PHONEINFO,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "logs phone information to file"
                 },
                 {
                  "writesecurityphoneinfo",
                  "wsp",
                  FC_WRITE_SECURITY_PHONEINFO,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "logs security phone information to file"
                 },
                 {
                  "lpt",
                  "l",
                  FC_LPT,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "sets which LPTx port to use"
                 },
                 {
                  "com",
                  "c",
                  FC_COM,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "sets which COMx port to use"
                 },
                 {
                  "erase",
                  "e",
                  FC_ERASE,
                  {FCAT_NUMBER,FCAT_NUMBER},
                  2,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "erases given area of the phone"
                 },
                 {
                  "eraseall",
                  "ea",
                  FC_ERASEALL,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "erases the phone"
                 },
                 {
                  "confirm",
                  "",
                  FC_CONFIRM,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "confirms erase operations"
                 },
                 {
                  "readfullflash",
                  "rff",
                  FC_READ_FULLFLASH,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads full flash into KNK file"
                 },
                 {
                  "rawreadfullflash",
                  "rrff",
                  FC_RAW_READ_FULLFLASH,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "reads full flash into FLS file"
                 },
                 {
                  "emulate",
                  "em",
                  FC_EMULATE,
                  {FCAT_STRING},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "emulates communication from KNK"
                 },
                 {
                  "rawemulate",
                  "rem",
                  FC_RAW_EMULATE,
                  {FCAT_STRING},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "emulates communication from FLS"
                 },
                 {
                  "checkchecksums",
                  "cc",
                  FC_CHECK_CHECKSUMS,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "checks checksums"
                 },
                 {
                  "fixchecksums",
                  "fc",
                  FC_FIX_CHECKSUMS,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "fixes checksums"
                 },
                 {
                  "checkimei",
                  "ci",
                  FC_CHECK_IMEI,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "checks IMEI"
                 },
                 {
                  "fiximei",
                  "fi",
                  FC_FIX_IMEI,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "fixes IMEI"
                 },
                 {
                  "checkseccode",
                  "cs",
                  FC_CHECK_SECCODE,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "checks security code"
                 },
                 {
                  "fixseccode",
                  "fs",
                  FC_FIX_SECCODE,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "fixes security code"
                 },
                 {
                  "clearseccodecounter",
                  "csc",
                  FC_CLEAR_SECCODE_COUNTER,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  TRUE,
                  TRUE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "clears security code counter"
                 },
                 {
                  "safeflashing",
                  "safe",
                  FC_SAFEFLASHING,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  FALSE,
                  "switches on safeflashing"
                 },
                 {
                  "resetsettings",
                  "rs",
                  FC_RESETSETTINGS,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "resets phone settings"
                 },
                 {
                  "resetphone",
                  "rp",
                  FC_RESETPHONE,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "resets phone"
                 },
                 {
                  "setsimlock",
                  "ssl",
                  FC_SETSIMLOCK,
                  {FCAT_NUMBER,FCAT_STRING,FCAT_NUMBER},
                  3,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "sets simlock data"
                 },
                 {
                  "converttoknk",
                  "ctk",
                  FC_CONVERTTOKNK,
                  {FCAT_STRING,FCAT_NUMBER},
                  2,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "converts input file to a KNK file"
                 },
                 {
                  "setcontrast",
                  "sc",
                  FC_SETCONTRAST,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "sets contrast of phone"
                 },
                 {
                  "setfaid",
                  "sf",
                  FC_SET_FAID,
                  {FCAT_NUMBER},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "sets FAID with new checksum"
                 },
                 {
                  "convertwttofls",
                  "cwttf",
                  FC_CONVERTWTTOFLS,
                  {FCAT_STRING,FCAT_STRING},
                  2,
                  TRUE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "converts input WinTesla file to an FLS file"
                 },
                 {
                  "convertwttoknk",
                  "cwttk",
                  FC_CONVERTWTTOKNK,
                  {FCAT_STRING,FCAT_STRING},
                  2,
                  TRUE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "converts input WinTesla file to an KNK file"
                 },
                 {
                  "resetsplock",
                  "rspl",
                  FC_RESETSPLOCK,
                  {FCAT_NONE},
                  0,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "resets SP lock data of phone"
                 },
                 {
                  "setimei",
                  "si",
                  FC_SETIMEI,
                  {FCAT_STRING},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  TRUE,
                  FALSE,
                  TRUE,
                  "sets new IMEI of phone"
                 },
		 {
                  "uploadcode",
                  "uc",
                  FC_UPLOADCODE,
                  {FCAT_STRING},
                  1,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  FALSE,
                  "upload arbitrary code (path)"
                 },
               };

#define CMDLINE_MAX_ARGLEN SYS_MAX_PATHNAME_LEN

typedef struct {
                char value[CMDLINE_MAX_ARGLEN];
               } tCMDLINE_STRINGARG;

typedef struct {
                 UINT8 U8num_of_args;
                 UINT16 U16cmdidx;
                 char longcmd_str[CMDLINE_MAX_LONGCMD_LEN];
                 tCMDLINE_STRINGARG arg[CMDLINE_MAX_CMD_ARGS];
               } tCMDLINE_USED_COMMAND;

typedef struct {
                 UINT8 U8num_of_commands;
                 tCMDLINE_USED_COMMAND command[CMDLINE_MAX_COMMANDS];
               } tCMDLINE_USED_COMMANDS;

#define CMDLINE_MAX_EXPANDED_NUMBERS 32
/* If i give an arg like,
  0..6,9..10,12 it will mean

  0,1,2,3,4,5,6,9,10,12
  so the expanded numbers mean that
*/

#define CMDLINE_MAX_STRINGS 3

typedef struct {
                eCMDLINE_COMMAND cmd;
                BOOL Bneedflasher;
                BOOL Bneedfchk;
                BOOL Bneedinfo;
                BOOL Bneedmbus;
                char longcmd_str[CMDLINE_MAX_LONGCMD_LEN];
                UINT8 U8num_of_strargs;
                tCMDLINE_STRINGARG stringarg[CMDLINE_MAX_STRINGS]; /* if the arg is a string */
                UINT8 U8num_of_numargs;
                UINT32 U32numarg[CMDLINE_MAX_EXPANDED_NUMBERS];
               } tCMDLINE_RUNTIME_COMMAND;

typedef struct {
                UINT8 U8num_of_commands;
                tCMDLINE_RUNTIME_COMMAND command[CMDLINE_MAX_COMMANDS];
               } tCMDLINE_RUNTIME_COMMANDS;


#define INFOSTR(x) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x); mylog(errbuff,FALSE,FALSE);}
#define INFOSTR1(x,p1) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1); mylog(errbuff,FALSE,FALSE);}
#define INFOSTR2(x,p1,p2) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1,p2); mylog(errbuff,FALSE,FALSE);}
#define INFOSTR3(x,p1,p2,p3) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1,p2,p3); mylog(errbuff,FALSE,FALSE);}
#define INFOSTR4(x,p1,p2,p3,p4) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1,p2,p3,p4); mylog(errbuff,FALSE,FALSE);}

#define ERRORSTR(x) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x); mylog(errbuff,TRUE,TRUE);}
#define ERRORSTR1(x,p1) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1); mylog(errbuff,TRUE,TRUE);}
#define ERRORSTR2(x,p1,p2) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1,p2); mylog(errbuff,TRUE,TRUE);}
#define ERRORSTR3(x,p1,p2,p3) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1,p2,p3); mylog(errbuff,TRUE,TRUE);}
#define ERRORSTR4(x,p1,p2,p3,p4) {memset(errbuff,0,sizeof(errbuff)); sprintf(errbuff,x,p1,p2,p3,p4); mylog(errbuff,TRUE,TRUE);}

#endif
