/********************************************************************
*                                                                   *
* FILE NAME:                                                        *
*                                                                   *
* PURPOSE:                                                          *
*                                                                   *
* FILE REFERENCES:                                                  *
*                                                                   *
* Name:                                                             *
*                                                                   *
* I/O:                                                              *
*                                                                   *
* Description:                                                      *
*                                                                   *
* EXTERNAL VARIABLES:                                               *
*                                                                   *
* Source:  <      >                                                 *
*                                                                   *
* Name:                                                             *
*                                                                   *
* Type:                                                             *
*                                                                   *
* I/O:                                                              *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* EXTERNAL REFERENCES:                                              *
*                                                                   *
* Name:                                                             *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* ABNORMAL TERMINATION CONDITIONS, ERROR AND WARNING MESSAGES:      *
*                                                                   *
* ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:                           *
*                                                                   *
* NOTES:                                                            *
*                                                                   *
/********************************************************************
*                                                                   *
* FILE NAME:                                                        *
*                                                                   *
* PURPOSE:                                                          *
*                                                                   *
* FILE REFERENCES:                                                  *
*                                                                   *
* Name:                                                             *
*                                                                   *
* I/O:                                                              *
*                                                                   *
* Description:                                                      *
*                                                                   *
* EXTERNAL VARIABLES:                                               *
*                                                                   *
* Source:  <      >                                                 *
*                                                                   *
* Name:                                                             *
*                                                                   *
* Type:                                                             *
*                                                                   *
* I/O:                                                              *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* EXTERNAL REFERENCES:                                              *
*                                                                   *
* Name:                                                             *
*                                                                   *
* Description:                                                      *
*                                                                   *
*                                                                   *
* ABNORMAL TERMINATION CONDITIONS, ERROR AND WARNING MESSAGES:      *
*                                                                   *
* ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:                           *
*                                                                   *
* NOTES:                                                            *
*                                                                   *
* REQUIREMENTS/FUNCTIONAL SPECIFICATIONS REFERENCES:                *
*                                                                   *
* DEVELOPMENT HISTORY:                                              *
*                                                                   *
* Date:                                                             *
*                                                                   *
* Author:                                                           *
*                                                                   *
* Change Id:                                                        *
*                                                                   *
* Release:                                                          *
*                                                                   *
* Description of change:                                            *
*                                                                   *
********************************************************************/
#include "../include/config.h"

#include <Windows.h>
#include <WinSvc.h>
#include <string.h>
#include <stdio.h>
#include "../include/sysdef.h"

#include "../include/fls1pub.h"
#include "miscpub.h"

#include "portloc.h"
#include "portpub.h"

static void PortCloseDriver(void);
static void PortOpenServiceManager(void);
static void PortStopIOPortService(void);
static void PortRemoveIOPortDriver(void);
static void PortCloseServiceManager(void);
static void PortInstallIOPortDriver(void);
static void PortStartIOPortService(void);
static BOOL PortOpendriver(void);
BOOL PortIsRunningOnNT(void);

static BOOL PortResult;
static UINT32 dwLastError;
static BOOL BisNT;
static char DriverName[IOPORT_DRIVER_NAME_LEN];
static char DriverFileName[IOPORT_DRIVER_NAME_LEN];
static BOOL bRemoveDevice;
static BOOL bStopService;
static SC_HANDLE hManager;
static SC_HANDLE hDevice;

static SINT32 FileAge(const char *FileName)
{
	SINT32 Result = -1;
	HANDLE Handle;
	WIN32_FIND_DATA FindData;
	FILETIME LocalFileTime;
	UINT16 Hi, Lo;

	Handle = FindFirstFile(FileName, &FindData);

	if (Handle != INVALID_HANDLE_VALUE) {
		FindClose(Handle);
		if (!
		    (FindData.
		     dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
			FileTimeToLocalFileTime(&FindData.ftLastWriteTime,
						&LocalFileTime);
			FileTimeToDosDateTime(&LocalFileTime, &Hi, &Lo);
			Result = (UINT32) (Hi << 16) | Lo;
		}
	}

	return Result;
}

static BOOL FileExists(const char *FileName)
{
	return FileAge(FileName) != -1;
}

static BOOL ReturnDriverPath(const char *Drivername, char *ext,
			     char *Pathname)
{
	char buf[128];
	BOOL Result;

	GetCurrentDirectory(sizeof(buf) - 1, buf);
	strcat(buf, "\\");
	strcat(buf, Drivername);
	strcat(buf, ext);

	Result = FileExists(buf);

	if (!Result) {
		GetSystemDirectory(buf, MAX_PATH);
		strcat(buf, "\\");
		strcat(buf, Drivername);
		strcat(buf, ext);
		Result = FileExists(buf);
	}

	strcpy(Pathname, buf);
	return Result;
}

BOOL PortCreate(void)
{
	hDevice = (SC_HANDLE) INVALID_HANDLE_VALUE;
	hManager = 0;
	strcpy(DriverName, IOPORT_DRIVER_NAME);
	strcpy(DriverFileName, IOPORT_DRIVER_NAME);
	bRemoveDevice = TRUE;
	bStopService = TRUE;
	BisNT = PortIsRunningOnNT();
	PortResult = FALSE;
	dwLastError = 0;
	return PortOpendriver();
}

void PortDestroy(void)
{
	if (hDevice != INVALID_HANDLE_VALUE) {
		PortCloseDriver();
	}

	if (BisNT) {
		PortOpenServiceManager();
		if (hManager != 0) {
			if (bStopService) {
				PortStopIOPortService();
			}

			if (bRemoveDevice) {
				PortRemoveIOPortDriver();
			}
			PortCloseServiceManager();
		}
	}
}

static BOOL PortOpendriver(void)
{
	char buf[128];
	BOOL Result;

	Result = FALSE;
	strcpy(buf, "");
	if (hDevice != INVALID_HANDLE_VALUE) {
		PortCloseDriver();
	}

	if (!BisNT) {
		return TRUE;
	}

	PortOpenServiceManager();
	if (dwLastError == ERROR_SUCCESS) {
		PortInstallIOPortDriver();
		if (dwLastError != ERROR_SUCCESS) {
			if (dwLastError == ERROR_SERVICE_EXISTS) {
				bRemoveDevice = FALSE;
				dwLastError = ERROR_SUCCESS;
			} else {
				PortCloseServiceManager();
			}
		}

		if (dwLastError == ERROR_SUCCESS) {
			PortStartIOPortService();

			if (dwLastError != ERROR_SUCCESS) {
				if (dwLastError ==
				    ERROR_SERVICE_ALREADY_RUNNING) {
					bStopService = FALSE;
					dwLastError = ERROR_SUCCESS;
				}
			}

			PortCloseServiceManager();
			if (dwLastError == ERROR_SUCCESS) {
				strcpy(buf, "\\\\.\\");
				strcat(buf, DriverFileName);
				hDevice = (SC_HANDLE) CreateFile(buf,
								 GENERIC_READ
								 |
								 GENERIC_WRITE,
								 0, NULL,
								 OPEN_EXISTING,
								 FILE_ATTRIBUTE_NORMAL,
								 0);

				if (hDevice != INVALID_HANDLE_VALUE) {
					Result = TRUE;
				} else {
					dwLastError = GetLastError();
					Result = FALSE;
				}
			}	// dwLastError == ERROR_SUCCESS
		}		// dwLastError == ERROR_SUCCESS
	}			// dwLastError == ERROR_SUCCESS

	return Result;
}

static void PortCloseDriver(void)
{
	if (hDevice != INVALID_HANDLE_VALUE) {
		CloseHandle(hDevice);
		hDevice = (SC_HANDLE) INVALID_HANDLE_VALUE;
	}
}

BOOL PortIsOpen(void)
{
	return (hDevice != INVALID_HANDLE_VALUE);
}

static void PortOpenServiceManager(void)
{
	if (hManager == 0) {
		dwLastError = 0;
		hManager =
		    OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
		if (hManager == 0) {
			dwLastError = GetLastError();
			MDBGSTR1
			    ("PortOpenServiceManager : service manager could not be opened (%d)",
			     dwLastError);
		}
	}
}

static void PortCloseServiceManager(void)
{
	if (hManager != 0) {
		CloseServiceHandle(hManager);
		hManager = 0;
	}
}

static void PortRemoveIOPortDriver(void)
{
	SC_HANDLE hService;

	dwLastError = 0;
	hService = OpenService(hManager, DriverName, SERVICE_ALL_ACCESS);

	if (hService == 0) {
		dwLastError = GetLastError();
		MDBGSTR1
		    ("PortRemoveIOPortDriver : service does not exist (%d)",
		     dwLastError);
	} else {
		if (!DeleteService(hService)) {
			dwLastError = GetLastError();
			MDBGSTR1
			    ("PortRemoveIOPortDriver : could not delete service (%d)",
			     dwLastError);
		}
		CloseServiceHandle(hService);
	}
}

static void PortInstallIOPortDriver(void)
{
	SC_HANDLE hService;
	char DriverPath[128];

	dwLastError = ERROR_SUCCESS;
	if (!ReturnDriverPath(DriverFileName, ".sys", DriverPath)) {
		MDBGSTR1("PortInstallIOPortDriver : driver '%s' not found",
			 DriverFileName);
		dwLastError = !ERROR_SUCCESS;
		return;
	}
	// we don't have a running service
	hService = CreateService(hManager, DriverName, DriverName,
				 SERVICE_ALL_ACCESS,
				 SERVICE_KERNEL_DRIVER,
				 SERVICE_DEMAND_START,
				 SERVICE_ERROR_NORMAL,
				 DriverPath, NULL, NULL, NULL, NULL, NULL);

	if (hService == 0) {
		dwLastError = GetLastError();
		if (dwLastError == ERROR_SERVICE_EXISTS) {
			MDBGSTR
			    ("PortInstallIOPortDriver : service already exists");
			hService =
			    OpenService(hManager, DriverName,
					SERVICE_ALL_ACCESS);
			if (hService == 0) {
				dwLastError = GetLastError();
				MDBGSTR1
				    ("PortInstallIOPortDriver : cannot open service (%d)",
				     dwLastError);
				return;
			}
		}
	}

	/* we've created or opened the service */
	CloseServiceHandle(hService);
	dwLastError = ERROR_SUCCESS;
}

static void PortStartIOPortService(void)
{
	SC_HANDLE hService;
	SERVICE_STATUS Status;

	dwLastError = ERROR_SUCCESS;
	hService = OpenService(hManager, DriverName, SERVICE_ALL_ACCESS);
	if (hService != 0) {
		if (!StartService(hService, 0, NULL)) {
			dwLastError = GetLastError();
			MDBGSTR1
			    ("PortStartIOPortService : cannot start service (%d)",
			     dwLastError);
			if (dwLastError == ERROR_FILE_NOT_FOUND) {
				if (!QueryServiceStatus(hService, &Status)) {
					dwLastError = GetLastError();
					MDBGSTR1
					    ("PortStartIOPortService : cannot query status (%d)",
					     dwLastError);
				}

				MDBGSTR1
				    ("PortStartIOPortService : status is %s",
				     (Status.dwCurrentState ==
				      SERVICE_STOPPED ? "stopped" :
				      "not stopped"));

				if (Status.dwCurrentState !=
				    SERVICE_STOPPED) {
					if (!ControlService
					    (hService,
					     SERVICE_CONTROL_STOP,
					     &Status)) {
						dwLastError =
						    GetLastError();
						MDBGSTR1
						    ("PortStartIOPortService : cannot stop old service (%d)",
						     dwLastError);
					}
				}

				/* assume that old service is stopped */
				if (!DeleteService(hService)) {
					dwLastError = GetLastError();
					MDBGSTR1
					    ("PortStartIOPortService : could not delete old service (%d)",
					     dwLastError);
				}
			}
		}
		CloseServiceHandle(hService);
	} else {
		dwLastError = GetLastError();
		MDBGSTR1
		    ("PortStartIOPortService : cannot start service (%d)",
		     dwLastError);
	}
}

static void PortStopIOPortService(void)
{
	SC_HANDLE hService;
	SERVICE_STATUS Status;

	dwLastError = ERROR_SUCCESS;
	hService = OpenService(hManager, DriverName, SERVICE_ALL_ACCESS);
	if (hService != 0) {
		if (!ControlService
		    (hService, SERVICE_CONTROL_STOP, &Status)) {
			dwLastError = GetLastError();
			MDBGSTR1
			    ("PortStopIOPortService : cannot stop service (%d)",
			     dwLastError);
			return;
		}
		CloseServiceHandle(hService);
	} else {
		dwLastError = GetLastError();
		MDBGSTR1
		    ("PortStopIOPortService : service does not exists (%d)",
		     dwLastError);
	}
}

static BOOL PortIsRunningOnNT(void)
{
	OSVERSIONINFOA ver;

	ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
	GetVersionExA(&ver);
	return (ver.dwPlatformId == VER_PLATFORM_WIN32_NT);
}
