//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------
#include "../include/config.h"

#ifdef WIN32
#include <windows.h>
#include <io.h>
#endif
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
//#include <dir.h>

#include <sys/types.h>
//#include <dirent.h>
#include "../include/sysdef.h"

#include "../include/fls1pub.h"
#include "miscpub.h"
#include "mbuspub.h"
#include "fle5pub.h"
#include "fls1loc.h"
#include "portpub.h"

extern eFLASH_ERROR flash_errno;

static BOOL Bautoalias = FALSE;
static tFLS1_DELAYBLOCK fls1_defaultdelayblock;
static tFLS1_CONFIG fls1_config;
static tFLS1_ALGOENTRY foundalgoentry;
static tFLASH_PHONE_INFO phoneinfo;

static pGaugeInit GaugeInit = NULL;
static pGaugeDone GaugeDone = NULL;
static pGaugeUpdate GaugeUpdate = NULL;

eFLASH_ERROR flash_errno;
eDEBUG_LEVEL debug_level = DEBUG_LEVEL_HIGH;
eDEVICE_STATE device_state = DEVICE_STATE_NONE;

static UINT8 AU8iobuffer[16384];


static BOOL FLS1_SetLPTPort(UINT8 U8lptport, UINT16 * U16baseport);
#ifdef MBUS
static BOOL FLS1_SetCOMPort(UINT8 U8comport, UINT32 U32comdelay);
#endif
static BOOL FLS1_CalculateFCHK(UINT32 * pU32fchk);

BOOL FLS1_DummyWrite(void);

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/
#ifdef WIN32
#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason,
			 void *lpReserved)
{
	return 1;
}
#endif
//---------------------------------------------------------------------------

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SetDirectory(eFLASH_DIRECTORY dirtype, char *pathname)
{
	char path[SYS_MAX_PATHNAME_LEN];

	if (pathname == NULL || strlen(pathname) == 0) {
		strcpy(path, "./");
	} else {
		strcpy(path, pathname);
		if (path[strlen(path) - 1] != '/')
			strcat(path, "/");
	}

	switch (dirtype) {
	case FLASH_DIRECTORY_CFG:
		strcpy(fls1_config.directories.cfgdir, path);
		return TRUE;

	case FLASH_DIRECTORY_LOADER:
		strcpy(fls1_config.directories.loaderdir, path);
		return TRUE;

	default:
		break;
	}

	flash_errno = FE_INVALID_DIRTYPE;
	return FALSE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_FileToPath(eFLASH_DIRECTORY dirtype, char *filename,
			    char *pathname)
{
	switch (dirtype) {
	case FLASH_DIRECTORY_CFG:
		strcpy(pathname, fls1_config.directories.cfgdir);
		break;

	case FLASH_DIRECTORY_LOADER:
		strcpy(pathname, fls1_config.directories.loaderdir);
		break;

	default:
		flash_errno = FE_INVALID_DIRTYPE;
		return FALSE;
	}

	if (filename)
		strcat(pathname, filename);

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void FLS1_StoreBlock(tFLASH_BLOCK * block, UINT32 U32start,
			    UINT32 U32end)
{
	block->U32start = U32start;
	block->U32end = U32end;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_AddBlock(tFLASH_BLOCKS * blocks, UINT32 U32start,
			  UINT32 U32end)
{
	UINT8 *U8block = &blocks->U8num_of_blocks;

	if (*U8block >= FLASH_MAX_BLOCKS) {
		flash_errno = FE_CANT_ADD_BLOCK;
		return FALSE;
	}

	FLS1_StoreBlock(&blocks->block[*U8block], U32start, U32end);
	(*U8block)++;

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void FLS1_CreateParamBlock(UINT8 U8cmd, UINT32 U32startaddr,
				  UINT32 U32count, UINT8 U8chk,
				  UINT8 * pAU8cmdblock)
{
	UINT32 U32endaddr = U32startaddr + U32count - 1;
	UINT8 *pU8ptr = pAU8cmdblock + 1;
	UINT8 *pU8start = pU8ptr;

	*pAU8cmdblock = U8cmd;

	U32count = misc_swapdword(U32count);
	U32count >>= 8;		/* 24 bit */

	U32startaddr = misc_swapdword(U32startaddr);
	U32startaddr >>= 8;	/* 24 bit */

	U32endaddr = misc_swapdword(U32endaddr);
	U32endaddr >>= 8;	/* 24 bit */

	memcpy(pU8ptr, &U32startaddr, 3);
	pU8ptr += 3;

	if (U8cmd != FLS1_ERASE_CMD) {
		*(pU8ptr++) = U8chk;

		memcpy(pU8ptr, &U32count, 3);
		pU8ptr += 3;
	} else {
		memcpy(pU8ptr, &U32endaddr, 3);
		pU8ptr += 3;
	}

	*pU8ptr = misc_calculate8bitchecksum(pU8start, pU8ptr - pU8start);
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetAutoAlias(BOOL Bval)
{
	HDBGSTR1("FLS1_SetAutoAlias : setting autoalias %s",
		 (Bval ? "on" : "off"));
	Bautoalias = Bval;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GetAutoAlias(void)
{
	return Bautoalias;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetRedirection(eREDIRECTION_TYPE redirecttype)
{
	misc_setredirection(redirecttype);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetDebugLevel(eDEBUG_LEVEL level)
{
	if (level > DEBUG_LEVEL_HIGH)
		level = DEBUG_LEVEL_HIGH;

	if (level < DEBUG_LEVEL_NONE)
		level = DEBUG_LEVEL_NONE;

	debug_level = level;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

eDEBUG_LEVEL FLS1_GetDebugLevel(void)
{
	return debug_level;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_ConfigurateBootstrapDCT3(UINT8 U8fval1, UINT8 * pAU8buffer,
				   UINT16 U16size)
{
	UINT16 U16checksum;

	pAU8buffer[FLS1_BOOTSTRAP_PARAM_OFFSET] = U8fval1;
	U16checksum =
	    misc_calculate16bitchecksum(TRUE, TRUE, pAU8buffer,
					U16size / 2 - 2);
	LDBGSTR1("FLS1_ConfigurateBootstrapDCT3 : checksum is %04X",
		 U16checksum);
	memcpy(&pAU8buffer[U16size - 4], &U16checksum,
	       sizeof(U16checksum));
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GetFlashParameters(char *NMP, UINT8 * pU8fval1, UINT8 * pU8fval2,
			     char *bloader)
{
	FILE *rfile;
	BOOL Bfinished;
	char Snmp[FLASH_MAX_NMP_LEN + 1], Smssw[FLASH_MAX_MSSW_LEN + 1],
	    Sbloader[SYS_MAX_FILENAME_LEN], Sforcedflashsize[4 + 1],
	    Sfval1[4 + 1], Sfval2[4 + 1];
	SINT16 numinputfields;
	char bloadercfgpath[SYS_MAX_PATHNAME_LEN];
	UINT8 U8dummyfval1, U8dummyfval2;

	if (pU8fval1 == NULL)
		pU8fval1 = &U8dummyfval1;

	if (pU8fval2 == NULL)
		pU8fval2 = &U8dummyfval2;

	fls1_config.U8forcedflashsize = 0;

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_CFG, fls1_config.models_list, bloadercfgpath))
		return FALSE;

	rfile = fopen(bloadercfgpath, "rt");
	printf("%s\n", bloadercfgpath);
	if (!rfile) {
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	Bfinished = FALSE;
	while (!Bfinished) {
		memset(Smssw, 0, sizeof(Smssw));
		memset(Snmp, 0, sizeof(Snmp));
		memset(Sfval1, 0, sizeof(Sfval1));
		memset(Sfval2, 0, sizeof(Sfval2));
		memset(Sbloader, 0, sizeof(Sbloader));

		numinputfields =
		    fscanf(rfile, "%s %s %s %s %s %s\n", Smssw, Snmp,
			   Sbloader, Sfval1, Sfval2, Sforcedflashsize);
		Bfinished = numinputfields != 6 || strlen(Sbloader) == 0
		    || strlen(Smssw) == 0 || strlen(Snmp) == 0;

		if (!Bfinished
		    && (strstr(Smssw, NMP) || strstr(Snmp, NMP))) {
			fclose(rfile);
			*pU8fval1 = (UINT8) misc_hextodword(Sfval1);
			*pU8fval2 = (UINT8) misc_hextodword(Sfval2);
			fls1_config.U8forcedflashsize =
			    atoi(Sforcedflashsize);
			strcpy(NMP, Smssw);
			strcpy(fls1_config.NMP, Snmp);
			if (bloader)
				strcpy(bloader, Sbloader);
			MDBGSTR3
			    ("FLS1_GetFlashParameters : parameters found %s(%s) - %s",
			     Smssw, Snmp, Sbloader);
			return TRUE;
		}
	}

	fclose(rfile);
	return FALSE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GetFlashID(tFLS1_FLASHID * pflashid, tFLS1_FLASHID * pnewflashid,
		     UINT8 U8length)
{
	char formatstring[32];
	char flashid[32];
	FILE *rfile;
	BOOL Bfound;
	UINT16 linelen;
	char linebuff[100];
	char *pchar, *endptr;
	UINT32 U32newflashid;
	char devlist[SYS_MAX_PATHNAME_LEN];

	if (pflashid == NULL || pnewflashid == NULL)
		return FALSE;

	if (pflashid->U16manuid == 0 && pflashid->U16devid == 0)
		return TRUE;

	U8length *= 2;
	sprintf(formatstring, "%%%02dX%%%02dX", U8length, U8length);
	sprintf(flashid, formatstring, misc_swapword(pflashid->U16manuid),
		misc_swapword(pflashid->U16devid));

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_CFG, fls1_config.aliasid_list, devlist))
		return FALSE;

	rfile = fopen(devlist, "rt");
	if (!rfile) {
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	do {
		memset(linebuff, 0, sizeof(linebuff));
		fgets(linebuff, sizeof(linebuff), rfile);
		linelen = strlen(linebuff);
		if (linelen > 0) {
			Bfound =
			    strncasecmp(linebuff, flashid,
				     strlen(flashid)) == 0;
		}
	} while (!Bfound && linelen > 0);
	fclose(rfile);

	if (!Bfound) {
		flash_errno = FE_INVALID_FLASHID;
		return FALSE;
	}

	pchar = strstr(linebuff, FLS1_ALIAS_ID_TXT);
	if (pchar) {		/* we have a alias */
		pchar += strlen(FLS1_ALIAS_ID_TXT);
		U32newflashid = strtoul(pchar, &endptr, 16);
		pnewflashid->U16manuid =
		    misc_swapword(MY_HIWORD(U32newflashid));
		pnewflashid->U16devid =
		    misc_swapword(MY_LOWORD(U32newflashid));
	} else {
		*pnewflashid = *pflashid;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GetFlashSize(tFLS1_FLASHID * pflashid, UINT8 U8length,
		       UINT8 * pU8size)
{
	char formatstring[32];
	char flashid[32];
	FILE *rfile;
	BOOL Bfound;
	UINT16 linelen;
	char linebuff[100];
	char *pchar, *endptr;
	char devlist[SYS_MAX_PATHNAME_LEN];

	if (pflashid == NULL || pU8size == NULL)
		return FALSE;

	*pU8size = 0;

	if (pflashid->U16manuid == 0 && pflashid->U16devid == 0)
		return TRUE;

	U8length *= 2;
	sprintf(formatstring, "%%%02dX%%%02dX", U8length, U8length);
	sprintf(flashid, formatstring, misc_swapword(pflashid->U16manuid),
		misc_swapword(pflashid->U16devid));

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_CFG, fls1_config.aliasid_list, devlist))
		return FALSE;

	rfile = fopen(devlist, "rt");
	if (!rfile) {
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	do {
		memset(linebuff, 0, sizeof(linebuff));
		fgets(linebuff, sizeof(linebuff), rfile);
		linelen = strlen(linebuff);
		if (linelen > 0) {
			Bfound =
			    strncasecmp(linebuff, flashid,
				     strlen(flashid)) == 0;
		}
	} while (!Bfound && linelen > 0);
	fclose(rfile);

	if (!Bfound) {
		*pU8size = 0;	/* can't be detected */
		return TRUE;
	}

	*pU8size = 0;
	pchar = strstr(linebuff, FLS1_SIZE_ID_TXT);
	if (pchar) {		/* we have the size */
		pchar += strlen(FLS1_SIZE_ID_TXT);
		*pU8size = (UINT8) strtoul(pchar, &endptr, 10);
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_FindModel(BOOL Bfromphone, void *phoneram, char *MSSW,
		    SINT32 * S32version)
{
	UINT8 *pU8phoneram;
	BOOL Bfinished;

	if (device_state < DEVICE_STATE_INITIALIZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (Bfromphone) {
		pU8phoneram =
		    &((tFLS1_CONFIGURATIONRESPONSE_DCT3 *) phoneram)->
		    mcusw[0];
	} else
		pU8phoneram = (UINT8 *) phoneram;

	/* skip version */
	*S32version = misc_getMCUversion(pU8phoneram);
	if (*S32version == -1) {
		flash_errno = FE_INVALID_MCU_VERSION;
		return FALSE;
	}

	pU8phoneram = (UINT8 *) strchr((char *) pU8phoneram, '\n');
	if (!pU8phoneram) {
		flash_errno = FE_INVALID_MCU_VERSION;
		return FALSE;
	}

	pU8phoneram++;		/* skip \n */

	/* skip date */
	pU8phoneram = (UINT8 *) strchr((char *) pU8phoneram, '\n');
	if (!pU8phoneram) {
		flash_errno = FE_INVALID_MCU_VERSION;
		return FALSE;
	}

	pU8phoneram++;		/* skip \n */

	for (Bfinished = FALSE; !Bfinished; pU8phoneram++) {
		Bfinished = *pU8phoneram < ' ';
		if (!Bfinished)
			*(MSSW++) = *pU8phoneram;
	}

	*MSSW = 0;

	return TRUE;
}



/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralReceiveConfigurationResponseDCT3(UINT16 U16baseport,
						  tFLS1_DELAYBLOCK *
						  pdelayblock,
						  tFLS1_CONFIGURATIONRESPONSE_DCT3
						  * configuration)
{
	UINT8 U8id;
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];
	UINT8 U8char;
	UINT8 U8idx;
	BOOL Bsuccess;

	if (device_state < DEVICE_STATE_BOOTSTRAP_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		flash_errno = FE_NOT_IMPLEMENTED;
		return TRUE;
	}

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, &U8id, sizeof(U8id), 0, pdelayblock)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (U8id != FLS1_PHONERAM_ID_DCT3) {
		flash_errno = FE_INVALID_MESSAGE_ID;
		return FALSE;
	}

	memset(configuration, 0, sizeof(*configuration));

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, AU8buffer, 35, 0, pdelayblock)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	configuration->U8asicid = AU8buffer[24 - 24];	// 0 Asic ID

	memcpy(configuration->U8configid, &AU8buffer[25 - 24], sizeof(configuration->U8configid));	// Config ID

	memcpy((UINT8 *) & configuration->U16mcuchecksum, &AU8buffer[33 - 24], sizeof(configuration->U16mcuchecksum));	// Flash CSUM

	memcpy(configuration->U8projspec, &AU8buffer[35 - 24], sizeof(configuration->U8projspec));	// Proj.Spec.
	memcpy(configuration->U8spare, &AU8buffer[51 - 24], sizeof(configuration->U8spare));	// Spare

	U8idx = 0;
	do {
		if (!FLE5_ReceiveFromPhone
		    (U16baseport, &U8char, sizeof(U8char), 0, NULL)) {
			flash_errno = FE_CANT_READWRITE;
			return FALSE;
		}

		configuration->mcusw[U8idx++] = U8char;
	} while (U8idx <= sizeof(configuration->mcusw) && U8char != 0);

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, AU8buffer, 9, 0, pdelayblock)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	memcpy((UINT8 *) & configuration->flashid[0], AU8buffer, sizeof(configuration->flashid[0]));	// ManuID+DevID 1
	memcpy((UINT8 *) & configuration->flashid[1], &AU8buffer[sizeof(configuration->flashid[0])], sizeof(configuration->flashid[1]));	// ManuID+DevID 2
	configuration->U8checksum = AU8buffer[8];	// Checksum

	Bsuccess =
	    misc_check8bitchecksum((UINT8 *) configuration,
				   sizeof(*configuration));

	if (!Bsuccess) {
		HDBGSTR
		    ("FLS1_GeneralReceiveConfigurationResponseDCT3 : checksum error");
		flash_errno = FE_INVALID_CHECKSUM;
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ReceiveConfigurationResponseDCT3(tFLS1_CONFIGURATIONRESPONSE_DCT3
					   * configuration)
{
	return FLS1_GeneralReceiveConfigurationResponseDCT3(fls1_config.
							    U16baseport,
							    &fls1_defaultdelayblock,
							    configuration);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralSendBootstrapDCT3(UINT16 U16baseport,
				   char *bootstrapname,
				   UINT8 U8fval1,
				   UINT32 U32retrycount,
				   UINT32 U32delay5,
				   UINT32 U32delay6,
				   tFLS1_CONFIGURATIONRESPONSE_DCT3 *
				   configuration)
{
	tFLS1_DELAYBLOCK delayblock;
	BOOL Bsuccess;
	UINT16 U16size;
	FILE *rfile;
	char bootstrap[SYS_MAX_PATHNAME_LEN];

	if (device_state < DEVICE_STATE_INITIALIZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		flash_errno = FE_NOT_IMPLEMENTED;
		device_state = DEVICE_STATE_BOOTSTRAP_RUNNING;
		return TRUE;
	}

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay5 = U32delay5;
	delayblock.U32delay6 = U32delay6;

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_LOADER, bootstrapname, bootstrap))
		return FALSE;

	rfile = fopen(bootstrap, "rb");
	if (!rfile) {
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	memset(configuration, 0, sizeof(*configuration));

	memset(AU8iobuffer, 0, sizeof(AU8iobuffer));
	U16size =
	    fread(AU8iobuffer, sizeof(UINT8), sizeof(AU8iobuffer), rfile);
	fclose(rfile);

	FLS1_ConfigurateBootstrapDCT3(U8fval1, AU8iobuffer, U16size);

	FLE5_SetDefaultTimes(&delayblock);
	//realtimePriority();
	Bsuccess =
	    FLE5_InitialiseBootstrapDCT3(U16baseport, AU8iobuffer, U16size,
					 U32retrycount);
	//normalPriority();
	if (!Bsuccess)
		return FALSE;

	device_state = DEVICE_STATE_BOOTSTRAP_RUNNING;

	if (!FLS1_GeneralReceiveConfigurationResponseDCT3
	    (U16baseport, &delayblock, configuration)) {
		flash_errno = FE_INVALID_CHECKSUM;
		return FALSE;
	}
#ifdef SAVE_PHONERAM
	rfile = fopen("phoneram.bin", "wb");
	if (rfile) {
		fwrite((UINT8 *) configuration, sizeof(UINT8),
		       sizeof(*configuration), rfile);
		fclose(rfile);
	}
#endif

	LDBGSTR
	    ("FLS1_GeneralSendBootstrapDCT3 : bootstrap seems to be running");

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SendBootstrapDCT3(char *bootstrapname,
			    UINT8 U8fval1,
			    UINT32 U32retrycount,
			    tFLS1_CONFIGURATIONRESPONSE_DCT3 *
			    configuration)
{
	return FLS1_GeneralSendBootstrapDCT3(fls1_config.U16baseport,
					     bootstrapname, U8fval1,
					     U32retrycount,
					     fls1_defaultdelayblock.
					     U32delay5,
					     fls1_defaultdelayblock.
					     U32delay6, configuration);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_DetectSupportedAlgoCodes(char *algocodename,
				   UINT8 * pU8numentries,
				   tFLS1_FLASHID flashids[])
{
	UINT16 U16algoparamofs;
	FILE *rfile;
	UINT16 i;
	UINT8 *palgoparam;
	tFLS1_ALGOENTRY *palgoentry;
	UINT16 U16algoentriesofs;
	UINT16 U16size;
	char algocode[SYS_MAX_PATHNAME_LEN];

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_LOADER, algocodename, algocode))
		return FALSE;

	rfile = fopen(algocode, "rb");
	if (!rfile) {
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	memset(AU8iobuffer, 0, sizeof(AU8iobuffer));
	U16size =
	    fread(&AU8iobuffer[4], sizeof(UINT8), sizeof(AU8iobuffer),
		  rfile);
	fclose(rfile);

	memcpy(&U16algoparamofs, &AU8iobuffer[FLS1_ALGOPARAM_OFS],
	       sizeof(U16algoparamofs));
	U16algoparamofs = misc_swapword(U16algoparamofs);
	if (U16algoparamofs > U16size) {
		flash_errno = FE_INVALID_ALGOCODE;
		return FALSE;
	}

	palgoparam = &AU8iobuffer[U16algoparamofs + 4];

	memcpy(&U16algoentriesofs, &palgoparam[FLS1_ALGOENTRIES_OFS],
	       sizeof(U16algoentriesofs));
	U16algoentriesofs = misc_swapword(U16algoentriesofs);
	if (U16algoentriesofs > U16size) {
		flash_errno = FE_INVALID_ALGOCODE;
		return FALSE;
	}

	palgoentry =
	    (tFLS1_ALGOENTRY *) & AU8iobuffer[U16algoentriesofs + 4];

	*pU8numentries = palgoparam[FLS1_NUMALGOENTRIES_OFS];

	for (i = 0; i < *pU8numentries; i++, palgoentry++) {
		flashids[i].U16devid = palgoentry->flashid.U16devid;
		flashids[i].U16manuid = palgoentry->flashid.U16manuid;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/
BOOL FLS1_CreateFlashIDList(void)
{
#ifdef WIN32
	char tmpstr[SYS_MAX_EXTENSION_LEN + 8],
	    searchpath[SYS_MAX_PATHNAME_LEN], inpath[SYS_MAX_PATHNAME_LEN];
	UINT8 U8numflashids;
	tFLS1_FLASHID flashids[FLS1_MAX_FLASHIDS];
	int done;
	//struct ffblk ffblk;
	int handle;
	struct _finddata_t info;
	FILE *rfile;
	UINT8 i;

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_CFG, fls1_config.flashid_list, inpath))
		return FALSE;

	rfile = fopen(inpath, "wt");
	if (!rfile) {
		flash_errno = FE_CANT_CREATE_FILE;
		return FALSE;
	}

	if (!FLS1_FileToPath(FLASH_DIRECTORY_LOADER, NULL, inpath))
		return FALSE;

	strcpy(searchpath, inpath);
	strcpy(tmpstr, "*.");
	strcat(tmpstr, fls1_config.dct3_extension_id);
	strcat(searchpath, tmpstr);

	MDBGSTR1("FLS1_CreateFlashIDList : searching through %s",
		 searchpath);
	done = handle = _findfirst(searchpath, &info);
	while (done != -1) {
		if (!FLS1_DetectSupportedAlgoCodes
		    (info.name, &U8numflashids, flashids)) {
			if (flash_errno != FE_INVALID_ALGOCODE) {
				return FALSE;
			}
		} else {
			for (i = 0; i < U8numflashids; i++) {
				fprintf(rfile, UINT32_FORMAT " %s\n",
					(UINT32) (misc_swapword
						  (flashids[i].
						   U16manuid) << 16) |
					misc_swapword(flashids[i].
						      U16devid),
					info.name);
			}
		}

		done = _findnext(handle, &info);
	}
	_findclose(handle);
	fclose(rfile);

	return TRUE;
#else
	/* TODO */
#endif

}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SearchAlgoCode(char *conversion_file, BOOL Brecreateidlist,
			 UINT16 U16manuid, UINT16 U16devid,
			 char *algocodename, BOOL * Bnofile)
{
	char inpath[SYS_MAX_PATHNAME_LEN], indir[SYS_MAX_PATHNAME_LEN];
	FILE *rfile;
	BOOL Bfinished;
	char Sflashid[8 + 1], Sfloader[SYS_MAX_FILENAME_LEN];
	UINT32 U32flashid;
	SINT16 numinputfields;
	UINT32 U32givenflashid =
	    (UINT32) (misc_swapword(U16manuid) << 16) |
	    misc_swapword(U16devid);

	*Bnofile = FALSE;

	if (!FLS1_FileToPath(FLASH_DIRECTORY_CFG, NULL, indir)) {
		MDBGSTR("FLS1_SearchAlgoCode : cannot convert input");
		return FALSE;
	}

	strcpy(inpath, indir);
	strcat(inpath, conversion_file);
	MDBGSTR1("FLS1_SearchAlgoCode : searching flashid in %s", inpath);

	rfile = fopen(inpath, "rt");
	if (!rfile) {
		if (Brecreateidlist) {
			FLS1_CreateFlashIDList();
			rfile = fopen(inpath, "rt");
			if (!rfile) {
				MDBGSTR1
				    ("FLS1_SearchAlgoCode : cannot open %s",
				     inpath);
				*Bnofile = TRUE;
				flash_errno = FE_FILE_NOT_FOUND;
				return FALSE;
			}
		} else {
			MDBGSTR1("FLS1_SearchAlgoCode : cannot open %s",
				 inpath);
			*Bnofile = TRUE;
			flash_errno = FE_FILE_NOT_FOUND;
			return FALSE;
		}
	}

	if (fls1_config.Bemulate) {
		fclose(rfile);
		strcpy(algocodename, "");
		return TRUE;
	}

	Bfinished = FALSE;
	while (!Bfinished) {
		memset(Sflashid, 0, sizeof(Sflashid));
		memset(Sfloader, 0, sizeof(Sfloader));

		numinputfields =
		    fscanf(rfile, "%s %s\n", Sflashid, Sfloader);
		Bfinished = numinputfields != 2;
		if (!Bfinished) {
			U32flashid = misc_hextodword(Sflashid);
			if (U32givenflashid == U32flashid) {
				fclose(rfile);
				strcpy(algocodename, Sfloader);
				return TRUE;
			}
		}
	}

	fclose(rfile);

	flash_errno = FE_NOT_SUPPORTED_FLASHID;
	return FALSE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralSendAlgoCodeDCT3(UINT16 U16baseport,
				  char *algocodename,
				  UINT8 U8fval1,
				  UINT8 U8fval2,
				  tFLS1_CONFIGURATIONRESPONSE_DCT3 *
				  configuration,
				  tFLS1_ALGOENTRY * pfoundalgoentry,
				  UINT32 U32delay5, UINT32 U32delay6)
{
	FILE *rfile;
	UINT16 U16size;
	SINT16 S16algocodesize;
	tFLS1_FLASHID flashid[FLS1_NUM_FLASHIDS];
	UINT8 U8numalgoentries;
	UINT16 i;
	UINT8 *palgoparam;
	tFLS1_ALGOENTRY *palgoentry;
	UINT16 U16algoparamofs, U16algoentriesofs;
	BOOL Bfound;
	UINT16 U16chk;
	tFLS1_DELAYBLOCK delayblock;
	UINT32 U32flashid;
	BOOL Bsuccess;
	UINT8 U8flashsize, U8accumulatedsize;
	char algocode[SYS_MAX_PATHNAME_LEN];

	if (device_state < DEVICE_STATE_BOOTSTRAP_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		flash_errno = FE_NOT_IMPLEMENTED;
		device_state = DEVICE_STATE_ALGOCODE_RUNNING;
		return TRUE;
	}

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay5 = U32delay5;
	delayblock.U32delay6 = U32delay6;

	if (!FLS1_FileToPath
	    (FLASH_DIRECTORY_LOADER, algocodename, algocode))
		return FALSE;

	MDBGSTR1("FLS1_GeneralSendAlgoCodeDCT3 : algocode is %s",
		 algocode);

	rfile = fopen(algocode, "rb");
	if (!rfile) {
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	memset(pfoundalgoentry, 0, sizeof(*pfoundalgoentry));

	memset(AU8iobuffer, 0, sizeof(AU8iobuffer));
	S16algocodesize = U16size =
	    fread(&AU8iobuffer[4], sizeof(UINT8), sizeof(AU8iobuffer) - 4,
		  rfile);
	fclose(rfile);

	memcpy(&U16algoparamofs, &AU8iobuffer[FLS1_ALGOPARAM_OFS],
	       sizeof(U16algoparamofs));
	U16algoparamofs = misc_swapword(U16algoparamofs);
	if (U16algoparamofs > U16size) {
		flash_errno = FE_INVALID_ALGOCODE;
		return FALSE;
	}

	palgoparam = &AU8iobuffer[U16algoparamofs + 4];

#ifdef DUMP_ALGOPARAM
	rfile = fopen("algparam.bin", "wb");
	if (rfile) {
		fwrite(palgoparam, sizeof(UINT8), 64, rfile);
		fclose(rfile);
	}
#endif

	memcpy(&U16algoentriesofs, &palgoparam[FLS1_ALGOENTRIES_OFS],
	       sizeof(U16algoentriesofs));
	U16algoentriesofs = misc_swapword(U16algoentriesofs);
	if (U16algoentriesofs > U16size) {
		flash_errno = FE_INVALID_ALGOCODE;
		return FALSE;
	}

	palgoentry =
	    (tFLS1_ALGOENTRY *) & AU8iobuffer[U16algoentriesofs + 4];

	if (FLS1_GetAutoAlias()) {
		for (i = 0, U8accumulatedsize = 0; i < FLS1_NUM_FLASHIDS;
		     i++) {
			if (!FLS1_GetFlashID
			    (&configuration->flashid[i], &flashid[i], 2))
				return FALSE;

			if (!FLS1_GetFlashSize
			    (&flashid[i], 2, &U8flashsize))
				return FALSE;

			HDBGSTR3
			    ("FLS1_GeneralSendAlgoCodeDCT3 : size of flashid %04X%04X is %d",
			     flashid[i].U16manuid, flashid[i].U16devid,
			     U8flashsize);

			U8accumulatedsize += U8flashsize;
		}
	}

	MDBGSTR1("FLS1_GeneralSendAlgoCodeDCT3 : forced flash size is %d",
		 fls1_config.U8forcedflashsize);
	if (fls1_config.U8forcedflashsize > 0)
		U8accumulatedsize = fls1_config.U8forcedflashsize;

	fls1_config.U32flashendaddr = FLASH_MCU_START_ADDRESS + (UINT32) U8accumulatedsize *0x100000 / 8;	/* in megs */
	MDBGSTR1("FLS1_GeneralSendAlgoCodeDCT3 : flash end is "
		 UINT32_FORMAT, fls1_config.U32flashendaddr);

	U8numalgoentries = palgoparam[FLS1_NUMALGOENTRIES_OFS];

	for (i = 0, Bfound = FALSE; i < U8numalgoentries && !Bfound; i++) {
		Bfound =
		    (palgoentry->flashid.U16devid ==
		     configuration->flashid[0].U16devid
		     && palgoentry->flashid.U16manuid ==
		     configuration->flashid[0].U16manuid);

		if (!Bfound) {
			palgoentry++;
		}
	}

	if (!Bfound) {
		flash_errno = FE_NO_ALGOCODE;
		return FALSE;
	}

	memcpy(pfoundalgoentry, palgoentry, sizeof(*pfoundalgoentry));
	memcpy(&foundalgoentry, palgoentry, sizeof(foundalgoentry));	/* for ourselves */

	memcpy(&U32flashid, &configuration->flashid[0],
	       sizeof(U32flashid));
	memcpy(&palgoparam[FLS1_ALGOPARAM_FLASHID_OFS], &U32flashid,
	       sizeof(U32flashid));

	memcpy(&U32flashid, &configuration->flashid[1],
	       sizeof(U32flashid));
	memcpy(&palgoparam
	       [FLS1_ALGOPARAM_FLASHID_OFS + sizeof(U32flashid)],
	       &U32flashid, sizeof(U32flashid));

	palgoparam[FLS1_ALGOPARAM_FVAL1_OFS] = U8fval1;
	palgoparam[FLS1_ALGOPARAM_FVAL2_OFS] = U8fval2;

	U16size = misc_swapword(U16size);
	memcpy(&AU8iobuffer[2], &U16size, sizeof(U16size));

	S16algocodesize += 2;
	if (S16algocodesize - 4 > 0)
		U16chk =
		    misc_calculate16bitchecksum(TRUE, TRUE,
						&AU8iobuffer[4],
						S16algocodesize / 2 - 3);
	else
		U16chk = 0;

	LDBGSTR1("FLS1_GeneralSendAlgoCodeDCT3 : checksum is %04X",
		 U16chk);
	memcpy(&AU8iobuffer[S16algocodesize - 2], (UINT8 *) & U16chk,
	       sizeof(U16chk));

	Bsuccess =
	    FLE5_SendToPhone(U16baseport, &AU8iobuffer[2], S16algocodesize,
			     FLS1_ALGOCODE_CMD, &delayblock);
	if (!Bsuccess) {
		return FALSE;
	}

	LDBGSTR
	    ("FLS1_GeneralSendAlgoCodeDCT3 : algo code seems to be running");
	return TRUE;
}



/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SendAlgoCodeDCT3(char *algocodename,
			   UINT8 U8fval1,
			   UINT8 U8fval2,
			   tFLS1_CONFIGURATIONRESPONSE_DCT3 *
			   configuration,
			   tFLS1_ALGOENTRY * pfoundalgoentry)
{
	return FLS1_GeneralSendAlgoCodeDCT3(fls1_config.U16baseport,
					    algocodename, U8fval1, U8fval2,
					    configuration, pfoundalgoentry,
					    fls1_defaultdelayblock.
					    U32delay5,
					    fls1_defaultdelayblock.
					    U32delay6);
}


/** 
 * upload arbitrary code after bootstrap initialisation - xx needs to be in seperate library 
 * if succesful, execution starts at the offset in [0x00000000]
 **/
BOOL FLS1_SendCodeDCT3(UINT8 * code,
		       size_t codesize, tFLS1_DELAYBLOCK * delayblock)
{
	UINT16 U16size;
	UINT16 U16chk;
	BOOL Bsuccess;
	UINT8 iobuffer[65536];

	//FLS1GetDefaultTimes(&delayblock);

	MDBGSTR1("SendCodeDCT3 : code size is %08x", codesize);

	/* copy the code */
	if (codesize > (sizeof(iobuffer) - 10)) {
		/* code too large */
		flash_errno = FE_TOO_MANY_ARGUMENTS;
		return FALSE;
	}

	/* zero initialisation */
	memset(iobuffer, 0, sizeof(iobuffer));

	/* copy supplied code segment */
	memcpy(&iobuffer[2], code, codesize);

	/* round up size to halfword aligned */
	U16size = (UINT16) ((codesize + 1) & 0xFFFE);
	//LDBGSTR1("SendCodeDCT3 : size is %04X", U16size);

	/* this doesn't make sense ... at all */
	U16chk =
	    misc_calculate16bitchecksum(FALSE, TRUE, &iobuffer[2],
					U16size / 2);

	LDBGSTR1("SendCodeDCT3 : checksum is %04X", U16chk);

	/* store (inverse) checksum at end, so halfword sum is 0 */
	iobuffer[2 + U16size + 0] = MY_HIBYTE(U16chk);
	iobuffer[2 + U16size + 1] = MY_LOBYTE(U16chk);
	U16size += 2;

	/* add two bytes (these are passed to the code in R0 */
	iobuffer[2 + U16size + 0] = 0;
	iobuffer[2 + U16size + 1] = 0;
	U16size += 2;

	iobuffer[0] = MY_HIBYTE(U16size);
	iobuffer[1] = MY_LOBYTE(U16size);

	//printf("%02X %02X %02X %02X %02X %02X %02X %02X\n",iobuffer[0],iobuffer[1],iobuffer[2],iobuffer[3],iobuffer[4],iobuffer[5],iobuffer[6],iobuffer[7]);

	U16chk =
	    misc_calculate16bitchecksum(FALSE, FALSE, &iobuffer[2],
					U16size / 2);
	if (U16chk != 0x0000) {
		printf("Huh - checksum erroneous %04X != 0\n", U16chk);
		return FALSE;
	}

	Bsuccess =
	    FLE5_SendToPhone(fls1_config.U16baseport, iobuffer,
			     U16size + 2, FLS1_ALGOCODE_CMD, delayblock);
	if (!Bsuccess) {
		return FALSE;
	}

	LDBGSTR("SendCodeDCT3 : code seems to be running");
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_PreparePhone(char *NMP, UINT32 U32retrycount)
{
	UINT8 U8fval1, U8fval2;
	char bloadername[SYS_MAX_FILENAME_LEN],
	    algocodename1[SYS_MAX_PATHNAME_LEN],
	    algocodename2[SYS_MAX_PATHNAME_LEN];
	tFLS1_ALGOENTRY algoentry;
	BOOL Bnomanualalgolist = FALSE;
	BOOL Bnofile;
	tFLS1_FLASHID flashid;
	SINT32 S32version;
	char newNMP[FLASH_MAX_NMP_LEN];

	if (device_state < DEVICE_STATE_INITIALIZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (!FLS1_GetFlashParameters(NMP, &U8fval1, &U8fval2, bloadername))
		return FALSE;

	MDBGSTR4
	    ("FLS1_PreparePhone : found flashing params %s(%d) - %02X, %02X",
	     NMP, fls1_config.U8forcedflashsize, U8fval1, U8fval2);

	fls1_config.U8fval1 = U8fval1;
	fls1_config.U8fval2 = U8fval2;

	if (!FLS1_SendBootstrapDCT3
	    (bloadername, U8fval1, U32retrycount, &fls1_config.config))
		return FALSE;

	memset(newNMP, 0, sizeof(newNMP));

	if (fls1_config.Bemulate) {
		/* we don't have a phone ram yet -> have to cheat a bit ;( */
		device_state = DEVICE_STATE_ALGOCODE_RUNNING;
		memset(&phoneinfo, 0, sizeof(phoneinfo));
		if (!FLS1_AnalyzeMCU
		    (&phoneinfo, newNMP, &phoneinfo.S32MCUversion)) {
			HDBGSTR("FLS1_PreparePhone : Invalid MCU");
			return FALSE;
		}
	} else {
		if (!FLS1_FindModel
		    (TRUE, &fls1_config.config, newNMP, &S32version)) {
			LDBGSTR
			    ("FLS1_PreparePhone : invalid NMP returned in phone RAM");
		} else {
			strcpy(NMP, newNMP);
		}
	}

	LDBGSTR1("FLS1_PreparePhone : detected NMP is %s", NMP);

	/* just for updating NMP in phone info */
	if (!FLS1_GetFlashParameters(NMP, NULL, NULL, NULL)) {
		LDBGSTR1("FLS1_PreparePhone : not supported NMP - %s",
			 newNMP);
		return FALSE;
	}

	/* converting to aliased */
	if (FLS1_GetAutoAlias()) {
		HDBGSTR("FLS1_PreparePhone : checking alias");
		if (!FLS1_GetFlashID
		    (&fls1_config.config.flashid[0], &flashid, 2))
			return FALSE;
		/* storing to original place */
		fls1_config.config.flashid[0] = flashid;
	} else {
		HDBGSTR("FLS1_PreparePhone : autoalias is switched off");
	}

	if (!fls1_config.Bemulate) {
		MDBGSTR2
		    ("FLS1_PreparePhone : searching algocode for %04X, %04X",
		     fls1_config.config.flashid[0].U16manuid,
		     fls1_config.config.flashid[0].U16devid);
		if (!FLS1_SearchAlgoCode
		    (fls1_config.flashid_list, TRUE,
		     fls1_config.config.flashid[0].U16manuid,
		     fls1_config.config.flashid[0].U16devid, algocodename1,
		     &Bnofile))
			return FALSE;

		MDBGSTR1
		    ("FLS1_PreparePhone : internally detected algocode - %s",
		     algocodename1);

		if (!FLS1_SearchAlgoCode
		    (fls1_config.algocode_list, FALSE,
		     fls1_config.config.flashid[0].U16manuid,
		     fls1_config.config.flashid[0].U16devid, algocodename2,
		     &Bnofile)) {
			if (!Bnofile) {	/* other error than not finding the manual algolist file */
				return FALSE;
			}
			Bnomanualalgolist = TRUE;
		}

		if (!Bnomanualalgolist) {
			MDBGSTR1
			    ("FLS1_PreparePhone : manually detected algocode - %s",
			     algocodename2);

			if (strcmp(algocodename1, algocodename2) != 0) {
				MDBGSTR
				    ("FLS1_PreparePhone : forcing manually detected algocode");
				strcpy(algocodename1, algocodename2);
			}
		}
	} else {
		strcpy(algocodename1, "");
	}

	if (!FLS1_SendAlgoCodeDCT3
	    (algocodename1, U8fval1, U8fval2, &fls1_config.config,
	     &algoentry))
		return FALSE;

	device_state = DEVICE_STATE_ALGOCODE_RUNNING;
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralProgrammingStatus(UINT16 U16baseport)
{
	UINT8 AU8statuscmd[] = { FLS1_GETSTATUS_CMD, -1 };
	BOOL Bsuccess;
	UINT8 U8id, U8err[2];
	UINT32 U32address;
	struct {
		UINT8 U8messageid;
		UINT8 U8errortype;
		UINT8 U8errorspec;
		UINT8 U8address[3];
		UINT8 U8expected;
		UINT8 U8contains;
		UINT8 U8checksum;
	} errinfo;

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		flash_errno = FE_NOT_IMPLEMENTED;
		return TRUE;
	}

	Bsuccess =
	    FLE5_SendToPhone(U16baseport, AU8statuscmd,
			     sizeof(AU8statuscmd), FLS1_CMDCODE1_CMD,
			     NULL);
	if (!Bsuccess) {
		return FALSE;
	}

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, &U8id, sizeof(U8id), 0, NULL)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (U8id != FLS1_PROGRAMSTATUS_ID) {
		flash_errno = FE_INVALID_PROGRAMMING_STATUS;
		return FALSE;
	}

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, &U8err[0], sizeof(U8err[0]), 0, NULL)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (U8err[0] == 0) {	/* good */
		if (!FLE5_ReceiveFromPhone
		    (U16baseport, &U8err[1], sizeof(U8err[0]), 0, NULL)) {
			flash_errno = FE_CANT_READWRITE;
			return FALSE;
		}

		if (!misc_check8bitchecksum(U8err, 2)) {
			flash_errno = FE_INVALID_STATUS_CHECKSUM;
			return FALSE;
		}

		return TRUE;	/* it's an error at a specified address */
	} else {
		errinfo.U8errortype = U8err[0];
		if (!FLE5_ReceiveFromPhone
		    (U16baseport, &errinfo.U8errorspec, 7, 2, NULL)) {
			flash_errno = FE_CANT_READWRITE;
			return FALSE;
		}

		if (!misc_check8bitchecksum(&errinfo.U8errortype, 8)) {
			flash_errno = FE_INVALID_STATUS_CHECKSUM;
			return FALSE;
		}

		U32address = 0;
		memcpy(&U32address, errinfo.U8address, 3);
		U32address <<= 8;
		fls1_config.status.U32address = misc_swapdword(U32address);
		fls1_config.status.U8expected = errinfo.U8expected;
		fls1_config.status.U8contains = errinfo.U8contains;
		MDBGSTR1("FLS1_GeneralProgrammingStatus : error at "
			 UINT32_FORMAT, fls1_config.status.U32address);
	}

	flash_errno = FE_INVALID_PROGRAMMING_STATUS;
	return FALSE;		/* it's an error */
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ProgrammingStatus(void)
{
	return FLS1_GeneralProgrammingStatus(fls1_config.U16baseport);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralMCURestart(UINT16 U16baseport)
{
	UINT8 AU8mcurestartcmd[] = { FLS1_MCURESET_CMD, -1 };
	tFLS1_DELAYBLOCK delayblock;

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		flash_errno = FE_NOT_IMPLEMENTED;
		return TRUE;
	}

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay1 *= 4;
	delayblock.U32delay4 *= 10;
	delayblock.U32delay5 *= 10;
	delayblock.U32delay6 *= 2;

	return FLE5_SendToPhone(U16baseport, AU8mcurestartcmd,
				sizeof(AU8mcurestartcmd),
				FLS1_CMDCODE2_CMD, &delayblock);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_MCURestart(void)
{
	return FLS1_GeneralMCURestart(fls1_config.U16baseport);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralReadConfigurationDCT3(UINT16 U16baseport,
				       tFLS1_CONFIGURATIONRESPONSE_DCT3 *
				       configuration)
{
	UINT8 AU8readconfigcmd[] = { FLS1_READCONFIG_CMD, -1 };
	tFLS1_DELAYBLOCK delayblock;
	BOOL Bsuccess;

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		flash_errno = FE_NOT_IMPLEMENTED;
		return TRUE;
	}

	FLE5_GetDefaultTimes(&delayblock);

	Bsuccess =
	    FLE5_SendToPhone(U16baseport, AU8readconfigcmd,
			     sizeof(AU8readconfigcmd), FLS1_CMDCODE2_CMD,
			     &delayblock);

	if (!Bsuccess) {
		return FALSE;
	}

	return FLS1_GeneralReceiveConfigurationResponseDCT3(U16baseport,
							    &delayblock,
							    configuration);
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ReadConfigurationDCT3(tFLS1_CONFIGURATIONRESPONSE_DCT3 *
				configuration)
{
	return FLS1_GeneralReadConfigurationDCT3(fls1_config.U16baseport,
						 configuration);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_OpenEmulatedFlashFile(FILE ** rfile, char *mode,
				       UINT32 U32seekaddr)
{
	long filepos;

	(*rfile) = fopen(fls1_config.emulatedflashfilename, mode);
	if (!(*rfile)) {
		HDBGSTR1
		    ("FLS1_OpenEmulatedFlashFile : cannot open emulation file %s",
		     fls1_config.emulatedflashfilename);
		return FALSE;
	}

	HDBGSTR1("FLS1_OpenEmulatedFlashFile :  binary emulation is %s",
		 (fls1_config.Bbinaryfileemulate ? "on" : "off"));

	/* must seek from current (after header) */
	if (!fls1_config.Bbinaryfileemulate)
		U32seekaddr += sizeof(tFILE_HEADER);

	HDBGSTR1("FLS1_OpenEmulatedFlashFile : seek address is %d",
		 U32seekaddr);
	fseek(*rfile, U32seekaddr, SEEK_CUR);

	/* save the file pointer position */
	filepos = ftell(*rfile);

	if ((UINT32) filepos != U32seekaddr) {
		HDBGSTR
		    ("FLS1_OpenEmulatedFlashFile : seek error in emulation file");
		flash_errno = FE_SEEK_ERROR;
		fclose(*rfile);
		return FALSE;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_ReadFlashBlockDCT3(UINT16 U16baseport,
				    UINT32 U32startaddr, UINT32 U32count,
				    UINT8 * pAU8buffer, UINT32 U32delay6)
{
	tFLS1_READWRITEPARAMS cmdblock;
	tFLS1_DELAYBLOCK delayblock;
	BOOL Bsuccess;
	UINT16 U16result;

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay2 *= 2;
	delayblock.U32delay3 *= 2;
	delayblock.U32delay6 = U32delay6;

	FLS1_CreateParamBlock(FLS1_READ_CMD, U32startaddr, U32count, 0,
			      (UINT8 *) & cmdblock);
	Bsuccess =
	    FLE5_SendToPhone(U16baseport, (UINT8 *) & cmdblock,
			     sizeof(cmdblock), FLS1_CMDCODE3_CMD,
			     &delayblock);
	if (!Bsuccess) {
		return FALSE;
	}

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, pAU8buffer, U32count, 0x02, &delayblock)) {
		MDBGSTR("FLS1_ReadFlashBlockDCT3 : cannot receive");
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (!FLE5_ReceiveFromPhone
	    (U16baseport, (UINT8 *) & U16result, sizeof(U16result), 0x02,
	     &delayblock)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (U16result != FLASH_READWRITE_RESULT_OK) {
		MDBGSTR1("FLS1_ReadFlashBlockDCT3 : result is %04X",
			 U16result);
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	/* Shit. This must be sent anyhow 
	   why? not true
	 */
	return TRUE;
	//return FLS1_DummyWrite();
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_EmulatedReadFlashBlockDCT3(UINT16 U16baseport,
					    UINT32 U32startaddr,
					    UINT32 U32count,
					    UINT8 * pAU8buffer,
					    UINT32 U32delay6)
{
	FILE *rfile;
	BOOL Bsuccess;

	if (U32delay6 || U16baseport) {	/* Warning removal */
	}

	if (!FLS1_OpenEmulatedFlashFile
	    (&rfile, "rb", U32startaddr - FLASH_MCU_START_ADDRESS)) {
		flash_errno = FE_EMUFILE_NOT_FOUND;
		return FALSE;
	}

	Bsuccess =
	    fread(pAU8buffer, sizeof(UINT8), U32count, rfile) == U32count;

	if (!Bsuccess) {
		HDBGSTR
		    ("FLS1_EmulatedReadFlashBlockDCT3 : cannot read in emulation file");
	}
	fclose(rfile);
	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_Read(UINT32 U32startaddr, UINT32 U32count, UINT8 * pAU8buffer)
{
	BOOL Bsuccess;
#ifdef SAVE_PHONE_BLOCK
	FILE *rfile;
#endif

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bemulate) {
		Bsuccess =
		    FLS1_EmulatedReadFlashBlockDCT3(fls1_config.
						    U16baseport,
						    U32startaddr, U32count,
						    pAU8buffer,
						    fls1_defaultdelayblock.
						    U32delay6);
	} else {
		Bsuccess =
		    FLS1_ReadFlashBlockDCT3(fls1_config.U16baseport,
					    U32startaddr, U32count,
					    pAU8buffer,
					    fls1_defaultdelayblock.
					    U32delay6);
	}

	if (!Bsuccess) {
		flash_errno = FE_CANT_READWRITE;
	} else {
#ifdef SAVE_PHONE_BLOCK
		rfile = fopen("flash.fls", "wb");
		if (rfile) {
			fwrite(pAU8buffer, sizeof(UINT8), U32count, rfile);
			fclose(rfile);
		}
#endif

		LDBGSTR2("FLS1_Read : read block (%d bytes) starting from "
			 UINT32_FORMAT, U32count, U32startaddr);
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_EraseFlashBlockDCT3(UINT16 U16baseport,
				     UINT32 U32startaddr, UINT32 U32count,
				     UINT32 U32delay3, UINT32 U32delay6)
{
	tFLS1_ERASEPARAMS cmdblock;
	tFLS1_DELAYBLOCK delayblock;
	BOOL Bsuccess;

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay3 = U32delay3 * 20;
	delayblock.U32delay6 = U32delay6;

	FLS1_CreateParamBlock(FLS1_ERASE_CMD, U32startaddr, U32count, 0,
			      (UINT8 *) & cmdblock);
	Bsuccess =
	    FLE5_SendToPhone(U16baseport, (UINT8 *) & cmdblock,
			     sizeof(cmdblock), FLS1_CMDCODE3_CMD,
			     &delayblock);
	if (!Bsuccess) {
		return FALSE;
	}

	return FLS1_GeneralProgrammingStatus(U16baseport);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_EmulatedEraseFlashBlockDCT3(UINT16 U16baseport,
					     UINT32 U32startaddr,
					     UINT32 U32count,
					     UINT32 U32delay3,
					     UINT32 U32delay6)
{
	FILE *rfile;
	UINT8 AU8buffer[1024];
	UINT32 U32left, i, U32toerase;
	BOOL Bsuccess;

	if (U16baseport || U32delay3 || U32delay6) {	/* Warning removal */
	}

	if (!FLS1_OpenEmulatedFlashFile
	    (&rfile, "rb+", U32startaddr - FLASH_MCU_START_ADDRESS)) {
		flash_errno = FE_EMUFILE_NOT_FOUND;
		return FALSE;
	}

	memset(AU8buffer, 0, sizeof(AU8buffer));
	U32left = U32count;
	for (i = U32startaddr; i < U32startaddr + U32count;
	     i += sizeof(AU8buffer), U32left -= sizeof(AU8buffer)) {
		U32toerase =
		    (U32left >
		     sizeof(AU8buffer) ? sizeof(AU8buffer) : U32left);
		Bsuccess =
		    fwrite(AU8buffer, sizeof(UINT8), U32toerase,
			   rfile) == U32toerase;
		if (!Bsuccess) {
			fclose(rfile);
			flash_errno = FE_CANT_ERASE;
			return FALSE;
		}
	}
	fclose(rfile);
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_Erase(UINT32 U32startaddr, UINT32 U32count)
{
	BOOL Bsuccess, Bdoit = TRUE;

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bsafeflashing) {
		Bdoit = (U32startaddr <= fls1_config.U32endofppm
			 && fls1_config.U32endofppm > 0);
	}

	if (Bdoit) {
		if (fls1_config.Bemulate) {
			Bsuccess =
			    FLS1_EmulatedEraseFlashBlockDCT3(fls1_config.
							     U16baseport,
							     U32startaddr,
							     U32count,
							     foundalgoentry.
							     U8dummy1,
							     fls1_defaultdelayblock.
							     U32delay6);
		} else {
			Bsuccess =
			    FLS1_EraseFlashBlockDCT3(fls1_config.
						     U16baseport,
						     U32startaddr,
						     U32count,
						     foundalgoentry.
						     U8dummy1,
						     fls1_defaultdelayblock.
						     U32delay6);
		}
	} else {
		LDBGSTR1
		    ("FLS1_Erase : safeflashing does not allow erasing "
		     UINT32_FORMAT, U32startaddr);
		Bsuccess = TRUE;
	}

	if (!Bsuccess) {
		flash_errno = FE_CANT_ERASE;
	} else {
		if (Bdoit) {
			LDBGSTR2
			    ("FLS1_Erase : erased block (%d bytes) starting from "
			     UINT32_FORMAT, U32count, U32startaddr);
		}
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_EraseAllFlashDCT3(UINT16 U16baseport, UINT32 U32delay3,
				   UINT32 U32delay6)
{
	UINT8 AU8eraseallcmd[] = { FLS1_ERASEALL_CMD, -1 };
	tFLS1_DELAYBLOCK delayblock;

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay3 = U32delay3 * 20;
	delayblock.U32delay6 = U32delay6;

	return FLE5_SendToPhone(U16baseport, (UINT8 *) & AU8eraseallcmd,
				sizeof(AU8eraseallcmd), FLS1_CMDCODE3_CMD,
				&delayblock);
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_EmulatedEraseAllFlashDCT3(UINT16 U16baseport,
					   UINT32 U32delay3,
					   UINT32 U32delay6)
{
	FILE *rfile;
	UINT8 AU8buffer[1024];
	UINT32 U32toerase, U32left, i;
	BOOL Bsuccess;

	if (U16baseport || U32delay3 || U32delay6) {	/* Warning removal */
	}

	if (!FLS1_OpenEmulatedFlashFile(&rfile, "rb+", 0)) {
		flash_errno = FE_EMUFILE_NOT_FOUND;
		return FALSE;
	}

	memset(AU8buffer, 0, sizeof(AU8buffer));
	U32left = fls1_config.U32flashendaddr - FLASH_MCU_START_ADDRESS;
	for (i = FLASH_MCU_START_ADDRESS; i < fls1_config.U32flashendaddr;
	     i += sizeof(AU8buffer), U32left -= sizeof(AU8buffer)) {
		U32toerase =
		    (U32left >
		     sizeof(AU8buffer) ? sizeof(AU8buffer) : U32left);
		Bsuccess =
		    fwrite(AU8buffer, sizeof(UINT8), U32toerase,
			   rfile) == U32toerase;
		if (!Bsuccess) {
			fclose(rfile);
			flash_errno = FE_CANT_ERASE;
			return FALSE;
		}
	}
	fclose(rfile);

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_EraseAll(void)
{
	BOOL Bsuccess, Bdoit = TRUE;

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bsafeflashing) {
		Bdoit = FALSE;
	}

	if (Bdoit) {
		if (fls1_config.Bemulate) {
			Bsuccess =
			    FLS1_EmulatedEraseAllFlashDCT3(fls1_config.
							   U16baseport,
							   5 *
							   foundalgoentry.
							   U8dummy1,
							   fls1_defaultdelayblock.
							   U32delay6);
		} else {
			Bsuccess =
			    FLS1_EraseAllFlashDCT3(fls1_config.U16baseport,
						   5 *
						   foundalgoentry.U8dummy1,
						   fls1_defaultdelayblock.
						   U32delay6);
		}
	} else {
		LDBGSTR
		    ("FLS1_EraseAll : safeflashing does not allow fullerasing");
		Bsuccess = TRUE;
	}

	if (!Bsuccess) {
		flash_errno = FE_CANT_ERASE;
	} else {
		if (Bdoit) {
			LDBGSTR("FLS1_EraseAll : erased all flash");
		}
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_WriteFlashBlockDCT3(UINT16 U16baseport,
				     UINT32 U32startaddr, UINT32 U32count,
				     UINT8 * pAU8buffer, UINT32 U32delay6,
				     BOOL Bgetstatus)
{
	tFLS1_READWRITEPARAMS cmdblock;
	tFLS1_DELAYBLOCK delayblock;
	BOOL Bsuccess;
	UINT32 U32cmd = 0x200;	/* hopefully it means writing words one-by-one */
	UINT8 U8chk;
	UINT8 U8zerocmd = 0x00;

	FLE5_GetDefaultTimes(&delayblock);
	delayblock.U32delay2 *= 2;
	delayblock.U32delay3 *= 2;
	delayblock.U32delay6 = U32delay6;

	U8chk = misc_calculate8bitchecksum(pAU8buffer, U32count) + 1;
	FLS1_CreateParamBlock(FLS1_WRITE_CMD, U32startaddr, U32count,
			      U8chk, (UINT8 *) & cmdblock);

	Bsuccess =
	    FLE5_SendToPhone(U16baseport, (UINT8 *) & cmdblock,
			     sizeof(cmdblock), FLS1_CMDCODE3_CMD,
			     &delayblock);
	if (!Bsuccess) {
		return FALSE;
	}

	if (!FLE5_SendToPhone
	    (U16baseport, pAU8buffer, U32count, U32cmd, &delayblock)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (!FLE5_SendToPhone
	    (U16baseport, &U8zerocmd, sizeof(U8zerocmd), FLS1_CMDCODE3_CMD,
	     &delayblock)) {
		flash_errno = FE_CANT_READWRITE;
		return FALSE;
	}

	if (Bgetstatus) {
		return FLS1_GeneralProgrammingStatus(U16baseport);
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_EmulatedWriteFlashBlockDCT3(UINT16 U16baseport,
					     UINT32 U32startaddr,
					     UINT32 U32count,
					     UINT8 * pAU8buffer,
					     UINT32 U32delay6)
{
	FILE *rfile;
	BOOL Bsuccess;

	if (U16baseport || U32delay6) {	/* Warning removal */
	}

	if (!FLS1_OpenEmulatedFlashFile
	    (&rfile, "rb+", U32startaddr - FLASH_MCU_START_ADDRESS)) {
		flash_errno = FE_EMUFILE_NOT_FOUND;
		return FALSE;
	}

	Bsuccess =
	    fwrite(pAU8buffer, sizeof(UINT8), U32count, rfile) == U32count;

	fclose(rfile);
	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralWrite(UINT32 U32startaddr, UINT32 U32count,
		       UINT8 * pAU8buffer, BOOL Bgetstatus)
{
	BOOL Bsuccess, Bdoit = TRUE;

	if (device_state < DEVICE_STATE_ALGOCODE_RUNNING) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (fls1_config.Bsafeflashing) {
		Bdoit = (U32startaddr <= fls1_config.U32endofppm
			 && fls1_config.U32endofppm > 0);
	}

	if (Bdoit) {
		if (fls1_config.Bemulate) {
			Bsuccess =
			    FLS1_EmulatedWriteFlashBlockDCT3(fls1_config.
							     U16baseport,
							     U32startaddr,
							     U32count,
							     pAU8buffer,
							     fls1_defaultdelayblock.
							     U32delay6);
		} else {
			Bsuccess =
			    FLS1_WriteFlashBlockDCT3(fls1_config.
						     U16baseport,
						     U32startaddr,
						     U32count, pAU8buffer,
						     fls1_defaultdelayblock.
						     U32delay6,
						     Bgetstatus);
		}
	} else {
		LDBGSTR1
		    ("FLS1_Write : safeflashing does not allow writing to "
		     UINT32_FORMAT, U32startaddr);
		Bsuccess = TRUE;
	}

	if (!Bsuccess) {
		flash_errno = FE_CANT_READWRITE;
	} else {
		if (Bdoit) {
			LDBGSTR2
			    ("FLS1_Write : written block (%d bytes) starting from "
			     UINT32_FORMAT, U32count, U32startaddr);
		}
	}

	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_Write(UINT32 U32startaddr, UINT32 U32count, UINT8 * pAU8buffer)
{
	return FLS1_GeneralWrite(U32startaddr, U32count, pAU8buffer, TRUE);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void FLS1_AdjustAddresses(UINT32 * U32startaddr,
				 UINT32 * U32endaddr)
{
	if (*U32startaddr > fls1_config.U32flashendaddr)
		*U32startaddr = fls1_config.U32flashendaddr;

	if (*U32endaddr > fls1_config.U32flashendaddr)
		*U32endaddr = fls1_config.U32flashendaddr;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_GeneralReadLarge(UINT32 U32startaddr, UINT32 U32count,
				  char *flashfile, BOOL Bbinary,
				  BOOL Bcheckonly, BOOL B32bitcheck,
				  UINT32 * U32checksum)
{
	FILE *rfile;
	UINT8 AU8buffer[4096];
	UINT32 U32endaddr, i, U32left = U32count, U32toread;
	eFILE_TYPE file_type;
	UINT16 U16checksum = 0;

	MDBGSTR2("FLS1_GeneralReadLarge : reading from " UINT32_FORMAT
		 " %d bytes", U32startaddr, U32count);
	if (Bcheckonly && U32checksum)
		*U32checksum = 0;

	if (!Bcheckonly) {
		if (Bbinary)
			strcat(flashfile, "." FLASH_FLS_EXTENSION);
		else
			strcat(flashfile, "." FLASH_KNK_EXTENSION);
	}

	U32endaddr = U32startaddr + U32count;
#ifdef FLASHBOUNDSCHECK
	FLS1_AdjustAddresses(&U32startaddr, &U32endaddr);
#endif
	U32count = U32endaddr - U32startaddr;

	if (U32startaddr == FLASH_MCU_START_ADDRESS &&
	    U32startaddr + U32count >= fls1_config.U32flashendaddr) {
		file_type = FILE_TYPE_FULL;
	} else {
		file_type = FILE_TYPE_PARTIAL;
	}

	if (!Bcheckonly) {
		if (flashfile) {
			for (i = 0; i < strlen(flashfile); i++)
				flashfile[i] = toupper(flashfile[i]);
		} else {
			flash_errno = FE_INVALID_FILENAME;
			return FALSE;
		}

		misc_checkfilename(flashfile);
		rfile = fopen(flashfile, "wb");
		if (!rfile) {
			flash_errno = FE_CANT_CREATE_FILE;
			return FALSE;
		}
	}

	if (!Bbinary && !Bcheckonly) {
		if (!misc_writeheader(rfile, file_type, U32startaddr)) {
			fclose(rfile);
			flash_errno = FE_CANT_WRITE_HEADER;
			return FALSE;
		}
	}
#ifdef FLASHBOUNDSCHECK
	if (U32startaddr + U32count > fls1_config.U32flashendaddr) {
		U32count = fls1_config.U32flashendaddr - U32startaddr;
	}
#endif
	//printf("%08X %08X\n",U32startaddr,U32endaddr);
	if (GaugeInit) {
		GaugeInit(U32startaddr, U32endaddr);
	}

	/* read to mem */
	for (i = U32startaddr; i < U32startaddr + U32count;
	     i += sizeof(AU8buffer), U32left -= sizeof(AU8buffer)) {
		U32toread =
		    (U32left >
		     sizeof(AU8buffer) ? sizeof(AU8buffer) : U32left);

		if (!FLS1_Read(i, U32toread, AU8buffer)) {
			fclose(rfile);
			if (GaugeDone) {
				GaugeDone();
			}
			return FALSE;
		}

		if (Bcheckonly) {
			if (B32bitcheck) {
				(*U32checksum) +=
				    misc_calculate32bitchecksum(FALSE,
								FALSE,
								AU8buffer,
								U32toread /
								4);
			} else {
				U16checksum +=
				    misc_calculate16bitchecksum(FALSE,
								FALSE,
								AU8buffer,
								U32toread /
								2);
			}
		}

		if (!Bcheckonly)
			fwrite(AU8buffer, sizeof(UINT8), U32toread, rfile);

		if (GaugeUpdate) {
			GaugeUpdate(i);
		}
	}

	if (GaugeDone) {
		GaugeDone();
	}

	if (!Bcheckonly)
		fclose(rfile);
	else {
		if (!B32bitcheck) {
			*U32checksum = U16checksum;
		}
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ReadLarge(UINT32 U32startaddr, UINT32 U32count,
		    char *flashfilename, BOOL Bbinary)
{
	return FLS1_GeneralReadLarge(U32startaddr, U32count, flashfilename,
				     Bbinary, FALSE, FALSE, NULL);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_EraseLarge(UINT32 U32startaddr, UINT32 U32endaddr)
{
	UINT32 i;
	UINT32 U32left, U32count;

	if ((U32startaddr & 0xFF0000) != U32startaddr) {	/* not a block start */
		flash_errno = FE_INVALID_STARTADDR;
		return FALSE;
	}

	if (GaugeInit) {
		GaugeInit(U32startaddr, U32endaddr);
	}

	U32left = U32endaddr - U32startaddr;
	for (i = U32startaddr; i < U32endaddr;
	     i += FLASH_ERASE_BLOCK_SIZE, U32left -=
	     FLASH_ERASE_BLOCK_SIZE) {
		U32count =
		    (U32left >
		     FLASH_ERASE_BLOCK_SIZE ? FLASH_ERASE_BLOCK_SIZE :
		     U32left);
		if (U32count > 0) {
			if (!FLS1_Erase(i, U32count)) {
				if (GaugeDone) {
					GaugeDone();
				}
				return FALSE;
			}

			if (GaugeUpdate) {
				GaugeUpdate(i);
			}
		}
	}

	if (GaugeDone) {
		GaugeDone();
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_WriteLarge(UINT32 U32extensionid, char *flashfile,
		     UINT32 U32startfile, UINT32 U32countfile,
		     UINT32 U32givenstartaddr, BOOL Bbinary)
{
	FILE *rfile;
	UINT8 AU8buffer[4096];
	UINT32 i, U32towrite;
	UINT32 U32left, U32len, U32end, U32startaddr, U32extid;
	eFILE_TYPE file_type;

	rfile = fopen(flashfile, "rb");
	if (!rfile) {
		MDBGSTR1("FLS1_WriteLarge : cannot open %s", flashfile);
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}
	

	//for (i = 0; i < strlen(flashfile); i++)
	//	flashfile[i] = toupper(flashfile[i]);

	MDBGSTR1("FLS1_WriteLarge : using %s", flashfile);
	/*
	if (U32extensionid != 0) {
		memcpy(&U32extid,
		       &flashfile[strlen(flashfile) - sizeof(U32extid)],
		       sizeof(U32extid));
		if (U32extid != U32extensionid) {
			MDBGSTR("FLS1_WriteLarge : invalid extension");
			flash_errno = FE_INVALID_EXTENSION;
			return FALSE;
		}
	}
	*/
	if (!Bbinary) {
		if (!misc_readheader(rfile, &file_type, &U32startaddr)) {
			MDBGSTR("FLS1_WriteLarge : invalid header");
			fclose(rfile);
			flash_errno = FE_CANT_READ_HEADER;
			return FALSE;
		}
	} else
		U32startaddr = U32givenstartaddr;
	if (U32startfile > 0)
		fseek(rfile, U32startfile, SEEK_CUR);
	if (U32countfile == 0)
		U32len = misc_getfilesize(rfile);
	else
		U32len = U32countfile;

	U32end = U32startaddr + U32len;
	MDBGSTR2("FLS1_WriteLarge : startaddress is " UINT32_FORMAT
		 " end is " UINT32_FORMAT, U32startaddr, U32end);

	FLS1_AdjustAddresses(&U32startaddr, &U32end);
	U32len = U32end - U32startaddr;

	MDBGSTR2("FLS1_WriteLarge : adjusted startaddress is "
		 UINT32_FORMAT " end is " UINT32_FORMAT, U32startaddr,
		 U32end);

	if (!fls1_config.Bskiperase) {
		if ((U32len % FLASH_ERASE_BLOCK_SIZE) != 0) {
			MDBGSTR("FLS1_WriteLarge : invalid erase size");
			flash_errno = FE_INVALID_FILESIZE;
			return FALSE;
		}

		if (!FLS1_EraseLarge(U32startaddr, U32end)) {
			MDBGSTR("FLS1_WriteLarge : cannot erase");
			return FALSE;
		}
	}

	if (GaugeInit) {
		GaugeInit(U32startaddr, U32end);
	}

	/* read from file mem */
	U32left = U32len;
	for (i = U32startaddr; i < U32end;
	     i += sizeof(AU8buffer), U32left -= sizeof(AU8buffer)) {
		U32towrite =
		    (U32left >
		     sizeof(AU8buffer) ? sizeof(AU8buffer) : U32left);
		fread(AU8buffer, sizeof(UINT8), U32towrite, rfile);

		if (U32towrite > 0) {
			if (!FLS1_Write(i, U32towrite, AU8buffer)) {
				fclose(rfile);
				if (GaugeDone) {
					GaugeDone();
				}
				return FALSE;
			}

			if (GaugeUpdate) {
				GaugeUpdate(i);
			}
		}
	}

	if (GaugeDone) {
		GaugeDone();
	}

	fclose(rfile);
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_Initialize(eDEBUG_LEVEL debug_level, pGaugeInit gaugeinit,
		     pGaugeDone gaugedone, pGaugeUpdate gaugeupdate,
		     pLogCallout logcallout,
		     eREDIRECTION_TYPE redirectiontype, char *logfile,
		     eFLASH_DELAY_TYPE delaytype, UINT32 U32basedelay,
		     UINT8 U8lptport, UINT8 U8comport, UINT32 U32comdelay,
		     BOOL Bsafeflash)
{
	BOOL Bsuccess;

	device_state = DEVICE_STATE_NONE;

	FLS1_SetAutoAlias(TRUE);

	FLS1_SetSafeFlashing(Bsafeflash);
	fls1_config.U32safeend = 0;
	FLS1_SetDebugLevel(debug_level);

	FLS1_SetLogCallout(logcallout);
	FLS1_SetGaugeInit(gaugeinit);
	FLS1_SetGaugeDone(gaugedone);
	FLS1_SetGaugeUpdate(gaugeupdate);
	/*
	   if (delaytype==FLASH_DELAY_TYPE_LOOP)
	   {
	   FLS1_SetLoopDelay();
	   FLS1_SetBaseDelay(U32basedelay);
	   }
	   else
	   {
	   misc_calibrate_delay();
	   FLS1_SetRdtscDelay();
	   }
	 */

#ifdef GENERAL_PORT
	if (!PortCreate()) {
		LDBGSTR("FLS1_Initialize : cannot initialize portdriver");
		flash_errno = FE_CANT_INIT_PORTDRIVER;
		return FALSE;
	}
#endif
	if (logfile && strlen(logfile) > 0
	    && redirectiontype != REDIRECTION_TYPE_NONE) {
		LDBGSTR1("FLS1_Initialize : redirecting log to %s",
			 logfile);
		FLS1_SetRedirection(redirectiontype);
		misc_startredirectionoferror(logfile);
	} else {
		FLS1_SetRedirection(REDIRECTION_TYPE_NONE);
	}
	FLS1_SetForcedFlashSize(0);
	FLS1_SetSkipErase(FALSE);
	FLS1_SetDirectory(FLASH_DIRECTORY_CFG, NULL);
	FLS1_SetDirectory(FLASH_DIRECTORY_LOADER, NULL);

	if (!FLS1_SetEmulation(NULL, TRUE)) {
		return FALSE;
	}
	FLS1_SetFlashIDList(FLASH_FLASHID_LIST);

	FLS1_SetAlgoCodeExtension(FLS_DCT3_EXTENSION_ID);
	FLS1_SetAliasIDList(FLASH_DEV_LIST);
	FLS1_SetModelsList(FLASH_MODELS_LIST);
	FLS1_SetAlgoCodeList(FLASH_ALGOCODE_LIST);

	FLE5_GetDefaultTimes(&fls1_defaultdelayblock);
	fls1_config.Bemulate = FALSE;

	if (!FLS1_SetLPTPort(U8lptport, &fls1_config.U16baseport)) {
		return FALSE;
	}
	FLE5_SetIdleState(fls1_config.U16baseport, 0);

#ifdef MBUS
	Bsuccess = FLS1_SetCOMPort(U8comport, U32comdelay);
#else
	Bsuccess = 1;
#endif
	if (Bsuccess) {
		device_state = DEVICE_STATE_INITIALIZED;
	}
	return Bsuccess;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GetMCUVersion(SINT32 * S32MCUversion, BOOL Bneedflasher,
			BOOL Bneedmbus)
{
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];
	SINT32 S32mcuptr;

	memset(AU8buffer, 0, sizeof(AU8buffer));

	if (Bneedflasher) {
		if (!FLS1_Read
		    (FLASH_MCU_START_ADDRESS, sizeof(AU8buffer),
		     AU8buffer))
			return FALSE;

		memcpy(&S32mcuptr, &AU8buffer[FLASH_MCU_PARAMBLOCK_OFFSET],
		       sizeof(S32mcuptr));

		S32mcuptr = misc_swapdword(S32mcuptr);
		if (S32mcuptr == -1) {
			flash_errno = FE_INVALID_MCU_PARAMBLOCK_PTR;
			return FALSE;
		}

		if (!FLS1_Read(S32mcuptr, sizeof(AU8buffer), AU8buffer))
			return FALSE;

		memcpy(&S32mcuptr, &AU8buffer[0], sizeof(S32mcuptr));
		S32mcuptr = misc_swapdword(S32mcuptr);
		if (S32mcuptr == -1) {
			flash_errno = FE_INVALID_MCU_PARAMBLOCK_PTR;
			return FALSE;
		}

		if (!FLS1_Read(S32mcuptr, sizeof(AU8buffer), AU8buffer))
			return FALSE;
	} 
#ifdef MBUS	
	else if (Bneedmbus) {	/* Use MBUS */
		if (!MBUS_GetVersion(AU8buffer, sizeof(AU8buffer)))
			return FALSE;
	}
#endif
	*S32MCUversion = misc_getMCUversion(AU8buffer);

	if (*S32MCUversion == -1) {
		MDBGSTR("FLS1_GetMCUVersion : invalid MCU version");
		flash_errno = FE_INVALID_MCU_VERSION;
	}

	return *S32MCUversion != -1;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_CheckHiddenMCUBlock(SINT32 S32start, SINT32 * S32end)
{
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];
	SINT32 S32ppmstart;
	SINT32 S32dest;
	BOOL Bfound;

	*S32end = S32start;

	S32ppmstart = (S32start & 0xFF0000) - 2;
	Bfound = FALSE;

	HDBGSTR1
	    ("FLS1_CheckHiddenMCUBlock : searching hidden MCU block until "
	     UINT32_FORMAT, fls1_config.U32flashendaddr);
	while (!Bfound
	       && (UINT32) S32ppmstart < fls1_config.U32flashendaddr) {
		if (!FLS1_Read(S32ppmstart, sizeof(AU8buffer), AU8buffer))
			return FALSE;

		/* must find "PPM" in the first some bytes */
		S32dest =
		    misc_memstr(AU8buffer, sizeof(AU8buffer),
				(UINT8 *) FLASH_PPM_ID,
				strlen(FLASH_PPM_ID));
		Bfound = S32dest < 2 && S32dest >= 0;
		if (!Bfound)
			S32ppmstart += 0x1000;	/* skip a whole block */
	}

	if (!Bfound) {
		INFO("FLS1_CheckHiddenMCUBlock : did not find PPM block. It might mean that phone has more flash chips on board. Try to overrule flashsize");
		flash_errno = FE_CANT_FIND_PPM;
		return FALSE;
	}

	*S32end = S32ppmstart + 1;
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_AnalyzeMCU(tFLASH_PHONE_INFO * phone_info, char *MSSW,
		     SINT32 * S32version)
{
	UINT8 AU8buffer[1024];
	SINT32 i;
	SINT32 S32realMCUend, S32start, S32end;
	UINT32 U32MCUheaderend, U32endofMCU = FLASH_MCU_START_ADDRESS;
	UINT16 U16originalMCUchecksum;
	BOOL Bgood, Btoread;
	UINT32 U32MCU2base, U32infobase;

	if (!FLS1_Read
	    (FLASH_MCU_START_ADDRESS, SYS_GENERAL_HEADER_LEN, AU8buffer)) {
		HDBGSTR("FLS1_AnalyzeMCU : cannot read block");
		return FALSE;
	}

	memcpy(&U32MCU2base, &AU8buffer[FLASH_MCU2_BASE_OFFSET],
	       sizeof(U32MCU2base));
	U32MCU2base = misc_swapdword(U32MCU2base);
	phone_info->U32MCU2base = U32MCU2base;

	memcpy(&U32MCUheaderend, &AU8buffer[FLASH_MCU_CHECKSUM_OFFSET1],
	       sizeof(U32MCUheaderend));
	U32MCUheaderend = misc_swapdword(U32MCUheaderend) >> 8;

	memcpy(&U16originalMCUchecksum,
	       &AU8buffer[FLASH_MCU_ORIGINAL_CHECKSUM1_OFFSET],
	       sizeof(U16originalMCUchecksum));
	U16originalMCUchecksum = misc_swapword(U16originalMCUchecksum);

	phone_info->U16mcuchecksum = U16originalMCUchecksum;

	phone_info->U8detectedfval1 = AU8buffer[FLASH_FVAL1_OFFSET];
	phone_info->U8detectedfval2 = AU8buffer[FLASH_FVAL2_OFFSET];

	if (!FLS1_AddBlock
	    (&phone_info->mcu_blocks, FLASH_MCU_START_ADDRESS,
	     U32MCUheaderend - 1)) {
		HDBGSTR("FLS1_AnalyzeMCU : cannot add block");
		return FALSE;
	}

	phone_info->BhiddenMCUblock = FALSE;

	i = 0;
	do {
		memcpy(&S32start,
		       &AU8buffer[i * 6 + FLASH_MCU_CHECKSUM_OFFSET1],
		       sizeof(S32start));
		memcpy(&S32end,
		       &AU8buffer[i * 6 + FLASH_MCU_CHECKSUM_OFFSET2],
		       sizeof(S32end));

		Bgood = S32start != -1 && S32end != -1;
		Btoread = Bgood;

		S32start = misc_swapdword(S32start) >> 8;
		S32end = misc_swapdword(S32end) >> 8;

		if (!Bgood) {	/* it might be the finish, although must check for hidden blocks */
			if (!FLS1_CheckHiddenMCUBlock
			    (U32endofMCU, &S32realMCUend)) {
				HDBGSTR
				    ("FLS1_AnalyzeMCU : cannot check hidden MCU block");
				return FALSE;
			}

			if ((UINT32) S32realMCUend != U32endofMCU) {	/* shit */
				S32start = U32endofMCU + 1;
				S32end = S32realMCUend;
				Btoread = TRUE;	/* must save the rest of the MCU */
				phone_info->BhiddenMCUblock = TRUE;
			}
		}

		if (Btoread) {
			U32endofMCU = (UINT32) S32end;

			if (!FLS1_AddBlock
			    (&phone_info->mcu_blocks, (UINT32) S32start,
			     (UINT32) S32end)) {
				HDBGSTR
				    ("FLS1_AnalyzeMCU : cannot add block");
				return FALSE;
			}
		}

		i++;
	} while (Bgood);

	if (!FLS1_Read(U32MCU2base, SYS_GENERAL_HEADER_LEN, AU8buffer)) {
		HDBGSTR("FLS1_AnalyzeMCU : cannot read");
		return FALSE;
	}

	memcpy(&U32infobase, AU8buffer, sizeof(U32infobase));	/* 24 bit address */
	U32infobase = misc_swapdword(U32infobase);
	phone_info->U32infobase = U32infobase;

	if (!FLS1_Read(U32infobase, SYS_GENERAL_HEADER_LEN, AU8buffer)) {
		HDBGSTR("FLS1_AnalyzeMCU : cannot read");
		return FALSE;
	}

	if (!FLS1_FindModel(FALSE, AU8buffer, MSSW, S32version)) {
		flash_errno = FE_CANT_FIND_VERSION;
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_AnalyzePPMBlock(tFLASH_PHONE_INFO * phone_info,
				 UINT32 U32start, UINT32 * U32blocklen,
				 BOOL * Blast)
{
	UINT8 AU8buffer[4096];
	UINT32 U32nextppmptr, U32ppmchecksum, U32count;
	UINT8 U8block_idx = phone_info->ppm_blocks.U8num_of_blocks;

	*Blast = FALSE;

	if (!FLS1_Read(U32start, SYS_GENERAL_HEADER_LEN, AU8buffer))
		return FALSE;

	memcpy(&U32nextppmptr, &AU8buffer[FLASH_NEXT_PPM_OFFSET],
	       sizeof(U32nextppmptr));
	U32nextppmptr = misc_swapdword(U32nextppmptr);
	memcpy(&U32ppmchecksum,
	       &AU8buffer[FLASH_PPM_BLOCK_CHECKSUM_OFFSET],
	       sizeof(U32ppmchecksum));
	U32ppmchecksum = misc_swapdword(U32ppmchecksum);
	memcpy(&U32count, &AU8buffer[FLASH_PPM_BLOCK_SIZE_OFFSET],
	       sizeof(U32count));
	U32count = misc_swapdword(U32count);

	phone_info->U32ppmchecksum[U8block_idx] = U32ppmchecksum;

	*U32blocklen = U32count;

	if (U32nextppmptr == 0) {
		*Blast = TRUE;
		return TRUE;
	}

	/* skip dumping checksum */

	if (!FLS1_AddBlock
	    (&phone_info->ppm_blocks, U32start, U32start + U32count - 1))
		return FALSE;

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_AnalyzePPM(tFLASH_PHONE_INFO * phone_info, BOOL Bneedfchk)
{
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];
	SINT32 S32MCUend;
	UINT32 U32ptr, U32start, U32blocklen;
	BOOL Bfound, Blast;
	UINT16 U16chk;

	if (phone_info->mcu_blocks.U8num_of_blocks == 0) {
		flash_errno = FE_CANT_FIND_MCU;
		return FALSE;
	}

	S32MCUend =
	    phone_info->mcu_blocks.block[phone_info->mcu_blocks.
					 U8num_of_blocks - 1].U32end + 1;

	if (!FLS1_Read(S32MCUend, sizeof(AU8buffer), AU8buffer))
		return FALSE;

	/* We have S32MCUend which is actually PPM's start */
	phone_info->Clanguage_pack =
	    AU8buffer[FLASH_PPM_LANGUAGE_PACK_OFFSET];

	U32start = S32MCUend + FLASH_PPM_LANGUAGE_PACK_PPM_DATA_OFFSET;

	phone_info->ppm_header_block.U32start = S32MCUend;
	phone_info->ppm_header_block.U32end = U32start - 1;

	Blast = FALSE;
	while (!Blast) {
		if (!FLS1_AnalyzePPMBlock
		    (phone_info, U32start, &U32blocklen, &Blast))
			return FALSE;

		U32start += U32blocklen;
	}

	if (phone_info->ppm_blocks.U8num_of_blocks != 0) {
		fls1_config.U32safeend =
		    phone_info->ppm_blocks.block[phone_info->ppm_blocks.
						 U8num_of_blocks -
						 1].U32end;
	} else {
		fls1_config.U32safeend =
		    phone_info->mcu_blocks.block[phone_info->mcu_blocks.
						 U8num_of_blocks -
						 1].U32end;
	}

	HDBGSTR1("FLS1_AnalyzePPM : safe end is " UINT32_FORMAT,
		 fls1_config.U32safeend);

	/* add half a megs to start searching backwards */
	U32ptr = (fls1_config.U32safeend & 0xFF0000) + 0x80000 - (sizeof(UINT16) + sizeof(UINT32));	/* check the 16 bit MCU file chk - phone_info->U16mcuchecksum */

	if (U32ptr > fls1_config.U32flashendaddr) {
		U32ptr = fls1_config.U32flashendaddr - (sizeof(UINT16) + sizeof(UINT32));	/* check the 16 bit MCU file chk - phone_info->U16mcuchecksum */
	}

	Bfound = FALSE;
	while (U32ptr > (UINT32) S32MCUend && !Bfound) {
		if (!FLS1_Read
		    (U32ptr, sizeof(UINT16) + sizeof(UINT32), AU8buffer))
			return FALSE;

		memcpy(&U16chk, AU8buffer, sizeof(U16chk));
		U16chk = misc_swapword(U16chk);

		Bfound = U16chk == phone_info->U16mcuchecksum;
		if (!Bfound) {
			U32ptr -= 0x10000;
		}
	}

	fls1_config.U32endofppm =
	    (Bfound ? U32ptr + sizeof(UINT16) +
	     sizeof(UINT32) : (fls1_config.U32safeend & 0xFF0000) +
	     0x10000);

	HDBGSTR1("FLS1_AnalyzePPM : end of PPM is " UINT32_FORMAT,
		 fls1_config.U32endofppm);

	phone_info->U32endofppm = fls1_config.U32endofppm;

	device_state = DEVICE_STATE_PPM_ANALYZED;

	if (Bneedfchk) {
		if (!FLS1_CalculateFCHK(&phone_info->U32fchk)) {
			return FALSE;
		}
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_AnalyzeEEPROM(tFLASH_PHONE_INFO * phone_info)
{
	tFLASH_BLOCKS *active_blocks;
	UINT8 i, j;
	UINT8 AU8buffer[FLASH_EEPROM_CHECKSUM_OFFSET2 +
			SYS_GENERAL_HEADER_LEN];
	UINT16 U16checksum;
	UINT32 U32flash_eeprom_checksum_offset1 =
	    FLASH_EEPROM_CHECKSUM_OFFSET1;
	UINT32 U32flash_eeprom_checksum_offset2 =
	    FLASH_EEPROM_CHECKSUM_OFFSET2;

	if (strstr(phone_info->MSSW, "NSE-5")) {	/* we have a 7110 */
		U32flash_eeprom_checksum_offset1 =
		    FLASH_EEPROM_NSE5_CHECKSUM_OFFSET1;
		U32flash_eeprom_checksum_offset2 =
		    FLASH_EEPROM_NSE5_CHECKSUM_OFFSET2;
	}

	active_blocks = &phone_info->eeprom_blocks;
	for (i = 0; i < active_blocks->U8num_of_blocks; i++) {
		if (FLS1_Read
		    (active_blocks->block[i].U32start, sizeof(AU8buffer),
		     AU8buffer) != TRUE)
			return FALSE;

		/* we have the start of the eeprom */
		memcpy(&U16checksum,
		       &AU8buffer[U32flash_eeprom_checksum_offset1],
		       sizeof(U16checksum));
		U16checksum = misc_swapword(U16checksum);
		phone_info->eeprom_info_blocks.block[i].U16checksum[0] =
		    U16checksum;

		memcpy(&U16checksum,
		       &AU8buffer[U32flash_eeprom_checksum_offset2],
		       sizeof(U16checksum));
		U16checksum = misc_swapword(U16checksum);
		phone_info->eeprom_info_blocks.block[i].U16checksum[1] =
		    U16checksum;

		for (j = 0; j < FLASH_EEPROM_IMEI_SIZE; j++) {
			phone_info->eeprom_info_blocks.block[i].IMEI[j] =
			    AU8buffer[FLASH_EEPROM_IMEI_OFFSET + j];
			phone_info->eeprom_info_blocks.block[i].
			    secIMEI[j] =
			    AU8buffer[FLASH_EEPROM_SECIMEI_OFFSET +
				      j] ^ 0x65;
		}

		for (j = 0; j < FLASH_EEPROM_SECCODE_SIZE; j++)
			phone_info->eeprom_info_blocks.block[i].
			    securitycode[j] =
			    AU8buffer[FLASH_EEPROM_SECCODE_OFFSET + j];
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_CheckIMEI(tFLASH_PHONE_INFO * phone_info, BOOL Bfix,
		    BOOL * Bchanged)
{
	tFLASH_BLOCKS *active_blocks;
	UINT8 i, j;
	UINT8 AU8buffer[FLASH_EEPROM_CHECKSUM_OFFSET2 +
			SYS_GENERAL_HEADER_LEN];
	BOOL Bdiffers = FALSE;

	*Bchanged = FALSE;

	if (device_state < DEVICE_STATE_PHONE_ANALYZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	active_blocks = &phone_info->eeprom_blocks;
	for (i = 0; i < active_blocks->U8num_of_blocks; i++) {
		if (!FLS1_Read
		    (active_blocks->block[i].U32start, sizeof(AU8buffer),
		     AU8buffer))
			return FALSE;

		for (j = 0; j < FLASH_EEPROM_IMEI_SIZE; j++) {
			if (AU8buffer[FLASH_EEPROM_SECIMEI_OFFSET + j] !=
			    (AU8buffer[FLASH_EEPROM_IMEI_OFFSET + j] ^
			     0x65)) {
				Bdiffers = TRUE;
				AU8buffer[FLASH_EEPROM_SECIMEI_OFFSET +
					  j] =
				    AU8buffer[FLASH_EEPROM_IMEI_OFFSET +
					      j] ^ 0x65;
			}
		}

		if (Bfix && Bdiffers) {
			*Bchanged = TRUE;

			if (!FLS1_Write(active_blocks->block[i].U32start, sizeof(AU8buffer), AU8buffer))	/* We're gonna write */
				return FALSE;
		}
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void FLS1_SecurityCodeToStr(UINT8 * pU8seccode, char *strseccode)
{
	UINT8 i;

	for (i = 0; i < FLASH_EEPROM_SECCODE_SIZE; i++)
		sprintf(&strseccode[2 * i], "%02X", pU8seccode[i]);

	strseccode[FLASH_PMM_SECCODE_SIZE - 1] = '\0';
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_CheckSecurityCode(tFLASH_PHONE_INFO * phone_info, BOOL Bfix,
			    BOOL * Bchanged)
{
	tFLASH_BLOCKS *active_blocks;
	UINT8 i;
	UINT16 j, k;
	UINT8 AU8buffer[FLASH_EEPROM_CHECKSUM_OFFSET2 +
			SYS_GENERAL_HEADER_LEN];
	UINT8 U8seccode_eep[FLASH_EEPROM_SECCODE_SIZE];
	char strseccode_eep[FLASH_PMM_SECCODE_SIZE];
	tFLASH_PMM_INFO *pmm_info;
	char *pU8;

	*Bchanged = FALSE;

	if (device_state < DEVICE_STATE_PHONE_ANALYZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	active_blocks = &phone_info->eeprom_blocks;
	for (i = 0; i < active_blocks->U8num_of_blocks; i++) {
		if (!FLS1_Read
		    (active_blocks->block[i].U32start, sizeof(AU8buffer),
		     AU8buffer))
			return FALSE;

		for (j = 0; j < FLASH_EEPROM_SECCODE_SIZE; j++)
			U8seccode_eep[j] =
			    AU8buffer[FLASH_EEPROM_SECCODE_OFFSET + j];

		FLS1_SecurityCodeToStr(U8seccode_eep, strseccode_eep);

		for (j = 0; j < phone_info->pmm_blocks.U8num_of_blocks;
		     j++)
			for (k = 0;
			     k <
			     phone_info->pmm_info_blocks.block[j].
			     U16num_of_entries; k++) {
				pmm_info =
				    &phone_info->pmm_info_blocks.block[j].
				    entry[k];
				if (pmm_info->U8id == FLASH_PMM_SECCODE_ENTRY_ID) {	/* gotcha */
					if (pmm_info->U16datalength !=
					    FLASH_PMM_SECCODE_SIZE) {
						flash_errno =
						    FE_INVALID_PMM_ENTRY_DATASIZE;
						return FALSE;
					}

					if (!FLS1_Read
					    (pmm_info->U32start,
					     sizeof(tFLASH_PMMENTRY) +
					     pmm_info->U16datalength,
					     AU8buffer))
						return FALSE;

					/* right after the header starts the seccode as string */
					pU8 =
					    (char *)
					    &AU8buffer[sizeof
						       (tFLASH_PMMENTRY)];
					if (strcmp(pU8, strseccode_eep) != 0 && Bfix) {	/* they differ */
						*Bchanged = TRUE;

						strcpy(pU8,
						       strseccode_eep);
						if (!FLS1_Write(pmm_info->U32start, sizeof(tFLASH_PMMENTRY) + pmm_info->U16datalength, AU8buffer))	/* We're gonna write */
							return FALSE;
					}
				}
			}
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ClearSecurityCodeCounter(tFLASH_PHONE_INFO * phone_info,
				   BOOL * Bchanged)
{
	tFLASH_BLOCKS *active_blocks;
	UINT8 i;
	UINT16 j, k;
	UINT8 AU8buffer[FLASH_EEPROM_CHECKSUM_OFFSET2 +
			SYS_GENERAL_HEADER_LEN];
	tFLASH_PMM_INFO *pmm_info;

	*Bchanged = FALSE;

	if (device_state < DEVICE_STATE_PHONE_ANALYZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	active_blocks = &phone_info->eeprom_blocks;
	for (i = 0; i < active_blocks->U8num_of_blocks; i++) {
		for (j = 0; j < phone_info->pmm_blocks.U8num_of_blocks;
		     j++)
			for (k = 0;
			     k <
			     phone_info->pmm_info_blocks.block[j].
			     U16num_of_entries; k++) {
				pmm_info =
				    &phone_info->pmm_info_blocks.block[j].
				    entry[k];
				if (pmm_info->U8id == FLASH_PMM_SECCODE_COUNTER_ENTRY_ID) {	/* gotcha */
					if (pmm_info->U16datalength != 1) {
						flash_errno =
						    FE_INVALID_PMM_ENTRY;
						return FALSE;
					}

					if (!FLS1_Read
					    (pmm_info->U32start,
					     sizeof(tFLASH_PMMENTRY) +
					     pmm_info->U16datalength,
					     AU8buffer))
						return FALSE;

					if (AU8buffer
					    [sizeof(tFLASH_PMMENTRY)] !=
					    0) {
						*Bchanged = TRUE;

						AU8buffer[sizeof
							  (tFLASH_PMMENTRY)]
						    = 0;

						if (!FLS1_Write(pmm_info->U32start, sizeof(tFLASH_PMMENTRY) + pmm_info->U16datalength, AU8buffer))	/* We're gonna write */
							return FALSE;
					}
				}
			}
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_AnalyzePMMBlock(tFLASH_PHONE_INFO * phone_info,
				 UINT8 U8pmmblockidx, UINT32 U32baseaddr,
				 UINT32 U32baseend)
{
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];	/* only header of PMM without data */
	tFLASH_PMMENTRY pmm_entry;
	UINT32 U32idx = FLASH_FIRST_PMM_OFFSET;
	BOOL Bfinished = FALSE;
	UINT16 i = 0;
	tFLASH_PMM_INFO *pmm_info;

	while (!Bfinished && U32idx < U32baseend - U32baseaddr) {
		if (!FLS1_Read
		    (U32baseaddr + U32idx, sizeof(pmm_entry), AU8buffer))
			return FALSE;

		memcpy(&pmm_entry, AU8buffer, sizeof(pmm_entry));
		pmm_entry.U16blockidx =
		    misc_swapword(pmm_entry.U16blockidx);
		pmm_entry.U16pmmbeginstructure =
		    misc_swapword(pmm_entry.U16pmmbeginstructure);
		pmm_entry.U16datachecksum =
		    misc_swapword(pmm_entry.U16datachecksum);
		pmm_entry.U16datalength =
		    misc_swapword(pmm_entry.U16datalength);
		pmm_entry.U16nextoffset =
		    misc_swapword(pmm_entry.U16nextoffset);

		if (pmm_entry.U16pmmbeginstructure == FLASH_PMM_END_ID) {
			phone_info->pmm_info_blocks.block[U8pmmblockidx].
			    U16num_of_entries = i;
			return TRUE;
		}

		if (pmm_entry.U16pmmbeginstructure !=
		    FLASH_PMM_BLOCKSTART_ID) {
			phone_info->pmm_info_blocks.block[U8pmmblockidx].
			    U16num_of_entries = i;
			return TRUE;
		} else {
			if (i >= FLASH_MAX_PMM_BLOCKS) {
				phone_info->pmm_info_blocks.
				    block[U8pmmblockidx].
				    U16num_of_entries = i;
				return TRUE;
			}

			pmm_info =
			    &phone_info->pmm_info_blocks.
			    block[U8pmmblockidx].entry[i++];

			if (pmm_entry.U8enablecode == FLASH_PMM_ENABLED)
				pmm_info->status =
				    FLASH_PMM_ENTRY_STATUS_ENABLED;
			else if (pmm_entry.U8enablecode ==
				 FLASH_PMM_DISABLED)
				pmm_info->status =
				    FLASH_PMM_ENTRY_STATUS_DISABLED;
			else
				pmm_info->status =
				    FLASH_PMM_ENTRY_STATUS_UNKNOWN;

			pmm_info->U32start = U32baseaddr + U32idx;
			pmm_info->U32end =
			    U32baseaddr + U32idx + sizeof(pmm_entry) +
			    pmm_entry.U16datalength - 1;
			pmm_info->U8id = pmm_entry.U8blocktype;
			pmm_info->U16datachecksum =
			    pmm_entry.U16datachecksum;
			pmm_info->U16datalength = pmm_entry.U16datalength;
		}

		U32idx = pmm_entry.U16nextoffset;
		Bfinished = U32idx == 0;
	}

	phone_info->pmm_info_blocks.block[U8pmmblockidx].
	    U16num_of_entries = i;
	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_AnalyzePMM(tFLASH_PHONE_INFO * phone_info)
{
	tFLASH_BLOCKS *active_blocks;
	UINT8 i;

	active_blocks = &phone_info->pmm_blocks;
	for (i = 0; i < active_blocks->U8num_of_blocks; i++) {
		if (!FLS1_AnalyzePMMBlock
		    (phone_info, i, active_blocks->block[i].U32start,
		     active_blocks->block[i].U32end))
			return FALSE;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static UINT32 FLS1_FindClosestStartAddress(UINT32 U32end,
					   tFLASH_BLOCKS * pmm_block,
					   tFLASH_BLOCKS * eeprom_block)
{
	UINT8 i;
	UINT32 U32currstart, U32start = 0, U32diff = U32end;

	for (i = 0; i < pmm_block->U8num_of_blocks; i++) {
		U32currstart = pmm_block->block[i].U32start;
		if (U32currstart > U32end
		    && U32currstart - U32end < U32diff) {
			U32diff = U32currstart - U32end < U32diff;
			U32start = U32currstart;
		}
	}

	for (i = 0; i < eeprom_block->U8num_of_blocks; i++) {
		U32currstart = eeprom_block->block[i].U32start;
		if (U32currstart > U32end
		    && U32currstart - U32end < U32diff) {
			U32diff = U32currstart - U32end < U32diff;
			U32start = U32currstart;
		}
	}

	if (U32start == 0)
		U32start = fls1_config.U32flashendaddr - 1;

	return U32start;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_Analyze(tFLASH_PHONE_INFO * phone_info)
{
	UINT32 i;
	UINT32 U32PPMend, U32block;
	UINT32 U32currentblockid;
	UINT8 AU8buffer[SYS_GENERAL_HEADER_LEN];
	UINT16 U16blockid;
	BOOL Bfoundblock;
	UINT32 U32end;
	tFLASH_BLOCKS *active_blocks;

	if (phone_info->ppm_blocks.U8num_of_blocks == 0) {
		if (phone_info->mcu_blocks.U8num_of_blocks == 0) {
			flash_errno = FE_CANT_FIND_MCU;
			return FALSE;
		}

		U32PPMend =
		    phone_info->mcu_blocks.block[phone_info->mcu_blocks.
						 U8num_of_blocks -
						 1].U32end + 1;
	} else
		U32PPMend =
		    phone_info->ppm_blocks.block[phone_info->ppm_blocks.
						 U8num_of_blocks -
						 1].U32end + 1;

	U32block = (U32PPMend & 0xFFFF0000);

	U32end = U32block + 0x100000;
	if (U32end > fls1_config.U32flashendaddr)
		U32end = fls1_config.U32flashendaddr;

	if (U32end < U32block) {
		flash_errno = FE_INVALID_FLASH_SIZE;
		return FALSE;
	}

	for (i = U32block; i < U32end;) {
		if (!FLS1_Read(i, sizeof(AU8buffer), AU8buffer)) {
			if (!fls1_config.Bemulate)
				return FALSE;
			else
				return TRUE;	/* probably EOF */
		}

		memcpy(&U32currentblockid, AU8buffer,
		       sizeof(U32currentblockid));
		U32currentblockid = misc_swapdword(U32currentblockid);

		Bfoundblock = FALSE;

		if (U32currentblockid == FLASH_BLOCK_ID) {	/* we have it */
			memcpy(&U16blockid, &AU8buffer[sizeof(UINT32)],
			       sizeof(UINT32));
			U16blockid = misc_swapword(U16blockid);
			switch (U16blockid) {
			case FLASH_BLOCK_ID_EEPROM:
				if (!FLS1_AddBlock
				    (&phone_info->eeprom_blocks, i, 0))
					return FALSE;

				i += 0x1000;
				Bfoundblock = TRUE;
				break;

			case FLASH_BLOCK_ID_PMM:
				if (!FLS1_AddBlock
				    (&phone_info->pmm_blocks, i, 0))
					return FALSE;

				i += 0x1000;
				Bfoundblock = TRUE;
				break;

			default:
				// not supported block id 
				break;
			}
		}

		if (!Bfoundblock)
			i += 0x1000;
	}

	active_blocks = &phone_info->pmm_blocks;
	for (i = 0; i < (UINT32) active_blocks->U8num_of_blocks; i++)
		active_blocks->block[i].U32end =
		    FLS1_FindClosestStartAddress(active_blocks->block[i].
						 U32start,
						 &phone_info->pmm_blocks,
						 &phone_info->
						 eeprom_blocks);

	active_blocks = &phone_info->eeprom_blocks;
	for (i = 0; i < (UINT32) active_blocks->U8num_of_blocks; i++)
		active_blocks->block[i].U32end =
		    FLS1_FindClosestStartAddress(active_blocks->block[i].
						 U32start,
						 &phone_info->pmm_blocks,
						 &phone_info->
						 eeprom_blocks);

	if (!FLS1_AnalyzeEEPROM(phone_info))
		return FALSE;

	if (!FLS1_AnalyzePMM(phone_info))
		return FALSE;

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GetPhoneInfo(tFLASH_PHONE_INFO * phone_info, char *MSSW,
		       BOOL Banalyze, BOOL Bneedflasher, BOOL Bneedmbus,
		       BOOL Bneedfchk)
{
	memset(phone_info->MSSW, 0, sizeof(phone_info->MSSW));
	strncpy(phone_info->MSSW, MSSW, sizeof(phone_info->MSSW));

	memset(phone_info->NMP, 0, sizeof(phone_info->NMP));
	strncpy(phone_info->NMP, fls1_config.NMP, sizeof(phone_info->NMP));

	if (!FLS1_GetMCUVersion
	    (&phone_info->S32MCUversion, Bneedflasher, Bneedmbus)) {
		MDBGSTR("FLS1_GetPhoneInfo : cannot get MCU version");
		return FALSE;
	}

	MDBGSTR("FLS1_GetPhoneInfo : MCU version is analyzed");

	if (Banalyze && Bneedflasher) {
		MDBGSTR("FLS1_GetPhoneInfo : Analyzing phone");

		if (!FLS1_AnalyzeMCU
		    (phone_info, phone_info->MSSW,
		     &phone_info->S32MCUversion))
			return FALSE;
		MDBGSTR("FLS1_GetPhoneInfo : MCU analyzed");

		if (!FLS1_AnalyzePPM(phone_info, Bneedfchk))
			return FALSE;
		MDBGSTR("FLS1_GetPhoneInfo : PPM analyzed");

		if (!FLS1_Analyze(phone_info))
			return FALSE;

		MDBGSTR("FLS1_GetPhoneInfo : PMM and EEPROM analyzed");

		memcpy(&phone_info->config, &fls1_config.config,
		       sizeof(phone_info->config));

		device_state = DEVICE_STATE_PHONE_ANALYZED;
	}

	return TRUE;
}



/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void FLS1_SaveByteStream(FILE ** rfile, UINT8 * pAU8buffer,
				UINT16 U16len)
{
	UINT16 i;

	for (i = 0; i < U16len; i++)
		fprintf(*rfile, "%02X", pAU8buffer[i]);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SavePhoneInfo(tFLASH_PHONE_INFO * phone_info, char *logfile,
			BOOL Bneedfchk)
{
	FILE *rfile;
	UINT8 i;
	UINT16 j;
	char strseccode[FLASH_PMM_SECCODE_SIZE];

	if (device_state < DEVICE_STATE_PHONE_ANALYZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	misc_checkfilename(logfile);
	rfile = fopen(logfile, "wt");
	if (!rfile) {
		flash_errno = FE_CANT_CREATE_FILE;
		return FALSE;
	}

	fprintf(rfile, "PHONE INFO by KNOK (build %d)\n\n", ENGINE_BUILD);

	fprintf(rfile, "MSSW: %s (%s)\n", phone_info->MSSW,
		phone_info->NMP);
	fprintf(rfile, "Software version: %02d.%02d\n",
		MY_HIBYTE(phone_info->S32MCUversion),
		MY_LOBYTE(phone_info->S32MCUversion));
	fprintf(rfile, "Language pack: %c\n", phone_info->Clanguage_pack);
	fprintf(rfile, "Detected phone settings: %02X, %02X\n",
		phone_info->U8detectedfval1, phone_info->U8detectedfval2);
	fprintf(rfile, "2nd MCU starts at: " UINT32_FORMAT "\n",
		phone_info->U32MCU2base);
	fprintf(rfile, "MCU information starts at: " UINT32_FORMAT "\n",
		phone_info->U32infobase);
	fprintf(rfile, "16bit MCU checksum: %04X\n",
		phone_info->U16mcuchecksum);
	if (Bneedfchk) {
		fprintf(rfile, "32bit PPM checksum: " UINT32_FORMAT,
			phone_info->U32fchk);
	}

	if (!fls1_config.Bemulate) {
		fprintf(rfile, "ASIC ID is: %02X\n",
			phone_info->config.U8asicid);
		fprintf(rfile, "Config ID is: ");
		FLS1_SaveByteStream(&rfile, phone_info->config.U8configid,
				    sizeof(phone_info->config.U8configid));
		fprintf(rfile,
			"\n16bit MCU checksum from phone RAM is: %04X\n",
			misc_swapword(phone_info->config.U16mcuchecksum));
		fprintf(rfile, "Project specification is: ");
		FLS1_SaveByteStream(&rfile, phone_info->config.U8projspec,
				    sizeof(phone_info->config.U8projspec));
		fprintf(rfile, "\nSpare ID is: ");
		FLS1_SaveByteStream(&rfile, phone_info->config.U8spare,
				    sizeof(phone_info->config.U8spare));
		fprintf(rfile, "\nPhone RAM checksum is : %02X\n",
			phone_info->config.U8checksum);
	}

	fprintf(rfile, "\nNumber of MCU blocks: %d\n",
		phone_info->mcu_blocks.U8num_of_blocks);
	if (phone_info->mcu_blocks.U8num_of_blocks > 0) {
		for (i = 0; i < phone_info->mcu_blocks.U8num_of_blocks;
		     i++)
			fprintf(rfile,
				"%sMCU block %d starts at " UINT32_FORMAT
				" ends at " UINT32_FORMAT "\n",
				(phone_info->BhiddenMCUblock
				 && i ==
				 phone_info->mcu_blocks.U8num_of_blocks -
				 1 ? "Hidden " : ""), i,
				phone_info->mcu_blocks.block[i].U32start,
				phone_info->mcu_blocks.block[i].U32end);
	}

	fprintf(rfile, "\nNumber of PPM blocks: %d\n",
		phone_info->ppm_blocks.U8num_of_blocks);
	if (phone_info->ppm_blocks.U8num_of_blocks > 0) {
		fprintf(rfile, "\nEnd of PPM is: " UINT32_FORMAT "\n",
			phone_info->U32endofppm);
		for (i = 0; i < phone_info->ppm_blocks.U8num_of_blocks;
		     i++)
			fprintf(rfile,
				"PPM block %d starts at " UINT32_FORMAT
				" ends at " UINT32_FORMAT
				", 32bit checksum is " UINT32_FORMAT "\n",
				i,
				phone_info->ppm_blocks.block[i].U32start,
				phone_info->ppm_blocks.block[i].U32end,
				phone_info->U32ppmchecksum[i]);
	}

	fprintf(rfile, "\nNumber of EEPROM blocks: %d\n",
		phone_info->eeprom_blocks.U8num_of_blocks);
	if (phone_info->eeprom_blocks.U8num_of_blocks > 0) {
		for (i = 0; i < phone_info->eeprom_blocks.U8num_of_blocks;
		     i++)
			fprintf(rfile,
				"EEPROM block %d starts at " UINT32_FORMAT
				" ends at " UINT32_FORMAT "\n", i,
				phone_info->eeprom_blocks.block[i].
				U32start,
				phone_info->eeprom_blocks.block[i].U32end);

		for (i = 0; i < phone_info->eeprom_blocks.U8num_of_blocks;
		     i++) {
			fprintf(rfile,
				"EEPROM block %d 16bit checksum 1 is %04X\n",
				i,
				phone_info->eeprom_info_blocks.block[i].
				U16checksum[0]);
			fprintf(rfile,
				"EEPROM block %d 16bit checksum 2 is %04X\n",
				i,
				phone_info->eeprom_info_blocks.block[i].
				U16checksum[1]);
		}

		for (i = 0; i < phone_info->eeprom_blocks.U8num_of_blocks;
		     i++) {
			fprintf(rfile, "IMEI in EEPROM block %d is: ", i);
			FLS1_SaveByteStream(&rfile,
					    phone_info->eeprom_info_blocks.
					    block[i].IMEI,
					    FLASH_EEPROM_IMEI_SIZE);

			fprintf(rfile,
				"\nSecurity IMEI in EEPROM block %d is: ",
				i);
			FLS1_SaveByteStream(&rfile,
					    phone_info->eeprom_info_blocks.
					    block[i].secIMEI,
					    FLASH_EEPROM_IMEI_SIZE);

			fprintf(rfile,
				"\nSecurity code in EEPROM block %d is: ",
				i);
			FLS1_SecurityCodeToStr(phone_info->
					       eeprom_info_blocks.block[i].
					       securitycode, strseccode);
			fprintf(rfile, "%s\n", strseccode);
		}
	}

	fprintf(rfile, "\nNumber of PMM blocks: %d\n",
		phone_info->pmm_blocks.U8num_of_blocks);
	if (phone_info->pmm_blocks.U8num_of_blocks > 0) {
		for (i = 0; i < phone_info->pmm_blocks.U8num_of_blocks;
		     i++) {
			fprintf(rfile,
				"PMM block %d starts at " UINT32_FORMAT
				" ends at " UINT32_FORMAT "\n", i,
				phone_info->pmm_blocks.block[i].U32start,
				phone_info->pmm_blocks.block[i].U32end);

			fprintf(rfile,
				"Number of entries in PMM block %d is: %d\n",
				i,
				phone_info->pmm_info_blocks.block[i].
				U16num_of_entries);
			for (j = 0;
			     j <
			     phone_info->pmm_info_blocks.block[i].
			     U16num_of_entries; j++) {
				fprintf(rfile,
					"\nID of entry %d is: %02X\n", j,
					phone_info->pmm_info_blocks.
					block[i].entry[j].U8id);
				fprintf(rfile, "Entry %d status is ", j);
				switch (phone_info->pmm_info_blocks.
					block[i].entry[j].status) {
				default:
				case FLASH_PMM_ENTRY_STATUS_UNKNOWN:
					fprintf(rfile, "unknown\n");
					break;

				case FLASH_PMM_ENTRY_STATUS_ENABLED:
					fprintf(rfile, "enabled\n");
					break;

				case FLASH_PMM_ENTRY_STATUS_DISABLED:
					fprintf(rfile, "disabled\n");
					break;
				}

				fprintf(rfile,
					"Entry %d starts at " UINT32_FORMAT
					"\n", j,
					phone_info->pmm_info_blocks.
					block[i].entry[j].U32start);
				fprintf(rfile,
					"Entry %d ends at " UINT32_FORMAT
					"\n", j,
					phone_info->pmm_info_blocks.
					block[i].entry[j].U32end);
				fprintf(rfile,
					"Length of data in entry %d is: %04X\n",
					j,
					phone_info->pmm_info_blocks.
					block[i].entry[j].U16datalength);
				fprintf(rfile,
					"16 bit checksum of data of entry %d is: %04X\n",
					j,
					phone_info->pmm_info_blocks.
					block[i].entry[j].U16datachecksum);
			}
		}
	}

	fclose(rfile);

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_SaveSecurityPhoneInfo(char *logfilename)
{
	return MBUS_SaveSecurityPhoneInfo(logfilename);
}
#endif


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_CheckPMMChecksums(tFLASH_PHONE_INFO * phone_info, BOOL Bfix)
{
	UINT8 AU8buffer[4096];	/* hopefully no PMM block will get bigger */
	UINT16 i, j;
	tFLASH_PMM_INFO *pmm_info;
	tFLASH_PMMENTRY pmm_entry, *pentry;
	UINT16 U16checksum;

	if (device_state < DEVICE_STATE_PHONE_ANALYZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (phone_info->pmm_blocks.U8num_of_blocks == 0)
		return TRUE;

	for (i = 0; i < phone_info->pmm_blocks.U8num_of_blocks; i++)
		for (j = 0;
		     j <
		     phone_info->pmm_info_blocks.block[i].
		     U16num_of_entries; j++) {
			pmm_info =
			    &phone_info->pmm_info_blocks.block[i].entry[j];
			if (!FLS1_Read
			    (pmm_info->U32start,
			     sizeof(tFLASH_PMMENTRY) +
			     pmm_info->U16datalength, AU8buffer))
				return FALSE;

			memcpy(&pmm_entry, AU8buffer, sizeof(pmm_entry));
			pmm_entry.U16datalength =
			    misc_swapword(pmm_entry.U16datalength);
			pmm_entry.U16datachecksum =
			    misc_swapword(pmm_entry.U16datachecksum);
			U16checksum =
			    misc_calculate8bitchecksumas16bit(&AU8buffer
							      [sizeof
							       (tFLASH_PMMENTRY)],
							      pmm_entry.
							      U16datalength);
			if (U16checksum != pmm_entry.U16datachecksum) {
				if (Bfix) {
					pentry =
					    (tFLASH_PMMENTRY *) &
					    AU8buffer[0];
					U16checksum =
					    misc_swapword(U16checksum);
					pentry->U16datachecksum =
					    U16checksum;

					/* Update checksum */
					if (!FLS1_Write(pmm_info->U32start, sizeof(AU8buffer), AU8buffer))	/* We're gonna write */
						return FALSE;
				}
			}
		}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_CheckChecksums(tFLASH_PHONE_INFO * phone_info, BOOL Bfix)
{
	UINT32 U32start, U32end;
	UINT16 U16checksum, i;
	UINT32 U32checksum;
	UINT8 AU8buffer[4096];
	UINT16 U16originalchecksum;
	tFLASH_BLOCK *flash_block;
	UINT32 U32flash_eeprom_checksum_offset1 =
	    FLASH_EEPROM_CHECKSUM_OFFSET1;
	UINT32 U32flash_eeprom_checksum_offset2 =
	    FLASH_EEPROM_CHECKSUM_OFFSET2;
	UINT16 U16flash_eeprom_endsum = FLASH_EEPROM_ENDSUM;

	if (device_state < DEVICE_STATE_PHONE_ANALYZED) {
		flash_errno = FE_INVALID_STATE;
		return FALSE;
	}

	if (!FLS1_CalculateFCHK(&phone_info->U32fchk)) {
		return FALSE;
	}

	if (phone_info->mcu_blocks.U8num_of_blocks != 0) {
		U32end = fls1_config.U32safeend;
		U32start = phone_info->mcu_blocks.block[1].U32start;

		if (!FLS1_Read
		    (fls1_config.U32endofppm - sizeof(UINT16) -
		     sizeof(UINT32) + 1, sizeof(U16originalchecksum),
		     (UINT8 *) & U16originalchecksum)) {
			return FALSE;
		}

		U16originalchecksum = misc_swapword(U16originalchecksum);

		if (!FLS1_GeneralReadLarge
		    (U32start, U32end - U32start + 1, NULL, FALSE, TRUE,
		     FALSE, &U32checksum)) {
			return FALSE;
		}

		if (U32checksum != (UINT32) phone_info->U16mcuchecksum
		    || U32checksum != (UINT32) U16originalchecksum) {
			if (Bfix) {
				if (!FLS1_Read
				    (FLASH_MCU_START_ADDRESS,
				     SYS_GENERAL_HEADER_LEN, AU8buffer)) {
					return FALSE;
				}

				U16checksum = (UINT16) U32checksum;
				U16checksum = misc_swapword(U16checksum);
				memcpy(&AU8buffer
				       [FLASH_MCU_ORIGINAL_CHECKSUM1_OFFSET],
				       &U16checksum, sizeof(U16checksum));

				/* Update checksum */
				if (!FLS1_Write(FLASH_MCU_START_ADDRESS, SYS_GENERAL_HEADER_LEN, AU8buffer))	/* We're gonna write */
					return FALSE;

				/* Update checksum */
				if (!FLS1_Write
				    (fls1_config.U32endofppm -
				     sizeof(UINT16) - sizeof(UINT32) + 1,
				     sizeof(U16checksum),
				     (UINT8 *) & U16checksum))
					return FALSE;
			}
		}
	}

	if (phone_info->ppm_blocks.U8num_of_blocks != 0) {
		for (i = 0; i < phone_info->ppm_blocks.U8num_of_blocks;
		     i++) {
			flash_block = &phone_info->ppm_blocks.block[i];
			U32start =
			    flash_block->U32start +
			    FLASH_PPM_BLOCK_HEADER_SIZE;
			U32end = flash_block->U32end;

			if (!FLS1_GeneralReadLarge
			    (U32start, U32end - U32start + 1, NULL, FALSE,
			     TRUE, TRUE, &U32checksum))
				return FALSE;

			if (U32checksum != phone_info->U32ppmchecksum[i]) {
				if (Bfix) {
					if (!FLS1_Read
					    (U32start,
					     SYS_GENERAL_HEADER_LEN,
					     AU8buffer))
						return FALSE;

					U32checksum =
					    misc_swapdword(U32checksum);
					memcpy(&AU8buffer
					       [FLASH_PPM_BLOCK_CHECKSUM_OFFSET],
					       &U32checksum,
					       sizeof(U32checksum));

					/* Update checksum */
					if (!FLS1_Write(U32start, SYS_GENERAL_HEADER_LEN, AU8buffer))	/* We're gonna write */
						return FALSE;
				}
			}
		}
	}

	if (phone_info->eeprom_blocks.U8num_of_blocks != 0) {
		if (strstr(phone_info->MSSW, "NSE-5")) {	/* we have a 7110 */
			U32flash_eeprom_checksum_offset1 =
			    FLASH_EEPROM_NSE5_CHECKSUM_OFFSET1;
			U32flash_eeprom_checksum_offset2 =
			    FLASH_EEPROM_NSE5_CHECKSUM_OFFSET2;
			U16flash_eeprom_endsum = FLASH_EEPROM_NSE5_ENDSUM;
		}

		for (i = 0; i < phone_info->eeprom_blocks.U8num_of_blocks;
		     i++) {
			flash_block = &phone_info->eeprom_blocks.block[i];
			U32start = flash_block->U32start;

			if (!FLS1_Read
			    (U32start + FLASH_EEPROM_CHECKSUM_START,
			     sizeof(AU8buffer), AU8buffer))
				return FALSE;

			U16checksum =
			    misc_calculate8bitchecksumas16bit(AU8buffer,
							      (UINT16)
							      (U32flash_eeprom_checksum_offset1
							       -
							       FLASH_EEPROM_CHECKSUM_START
							       - 1));

			/* check checksum 1 */
			if (U16checksum !=
			    phone_info->eeprom_info_blocks.block[i].
			    U16checksum[0]) {
				if (Bfix) {
					if (!FLS1_Read
					    (U32start, sizeof(AU8buffer),
					     AU8buffer))
						return FALSE;

					U16checksum =
					    misc_swapword(U16checksum);
					memcpy(&AU8buffer
					       [U32flash_eeprom_checksum_offset1],
					       &U16checksum,
					       sizeof(U16checksum));

					/* Update checksum */
					if (!FLS1_Write(U32start, sizeof(AU8buffer), AU8buffer))	/* We're gonna write */
						return FALSE;
				}
			}

			if (!FLS1_Read
			    (U32start + U32flash_eeprom_checksum_offset1 +
			     sizeof(UINT16), sizeof(AU8buffer), AU8buffer))
				return FALSE;

			U16checksum =
			    misc_calculate8bitchecksumas16bit(AU8buffer,
							      U32flash_eeprom_checksum_offset2
							      -
							      (U32flash_eeprom_checksum_offset1
							       +
							       sizeof
							       (UINT16)) -
							      1);
			U16checksum -= U16flash_eeprom_endsum;

			/* check checksum 2 */
			if (U16checksum !=
			    phone_info->eeprom_info_blocks.block[i].
			    U16checksum[1]) {
				if (Bfix) {
					if (!FLS1_Read
					    (U32start, sizeof(AU8buffer),
					     AU8buffer))
						return FALSE;

					U16checksum =
					    misc_swapword(U16checksum);
					memcpy(&AU8buffer
					       [U32flash_eeprom_checksum_offset2],
					       &U16checksum,
					       sizeof(U16checksum));

					/* Update checksum */
					if (!FLS1_Write(U32start, sizeof(AU8buffer), AU8buffer))	/* We're gonna write */
						return FALSE;
				}
			}
		}
	}

	return FLS1_CheckPMMChecksums(phone_info, Bfix);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SaveBlock(tFLASH_BLOCK * block, char *basepath, char *extrainfo,
		    BOOL Bbinary)
{
	char tmpfile[SYS_MAX_PATHNAME_LEN];

	strcpy(tmpfile, basepath);
	strcat(tmpfile, extrainfo);

	return FLS1_GeneralReadLarge(block->U32start,
				     block->U32end - block->U32start + 1,
				     tmpfile, Bbinary, FALSE, FALSE, NULL);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SaveBlocks(tFLASH_BLOCKS * flash_block, char *basepath,
		     char *mask, UINT8 U8num_of_args, UINT32 * U32numarg,
		     BOOL Bbinary)
{
	UINT8 i, idx;
	tFLASH_BLOCK *block;
	char tmpfile[SYS_MAX_PATHNAME_LEN], extrainfo[32];

	if (flash_block->U8num_of_blocks == 0)
		return TRUE;

	if (U8num_of_args == 0) {
		U8num_of_args = flash_block->U8num_of_blocks;
		for (i = 0; i < U8num_of_args; i++)
			U32numarg[i] = i;
	}

	for (i = 0; i < U8num_of_args; i++) {
		idx = (UINT8) U32numarg[i];
		if (idx < flash_block->U8num_of_blocks) {
			sprintf(extrainfo, mask, idx + 1,
				flash_block->U8num_of_blocks);
			strcpy(tmpfile, basepath);
			strcat(tmpfile, extrainfo);

			block = &flash_block->block[idx];

			if (!FLS1_GeneralReadLarge
			    (block->U32start,
			     block->U32end - block->U32start + 1, tmpfile,
			     Bbinary, FALSE, FALSE, NULL))
				return FALSE;
		}
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_DummyWrite(void)
{
	UINT8 AU8buffer[4];

	memset(AU8buffer, 0xFF, sizeof(AU8buffer));

	MDBGSTR("FLS1_DummyWrite : trying");
	return FLS1_GeneralWrite(FLASH_MCU_START_ADDRESS + 0x34, sizeof(AU8buffer), AU8buffer, FALSE);	/* We're gonna write */
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SetEmulation(char *filename, BOOL Bbinaryemulate)
{
	UINT8 j;
	UINT32 U32extid, U32len;
	FILE *rfile;
	UINT32 U32storedstartaddr;
	eFILE_TYPE file_type;
	BOOL Bsuccess;

	if (filename == NULL || strlen(filename) == 0) {
		fls1_config.Bemulate = FALSE;
		return TRUE;
	}

	HDBGSTR2
	    ("FLS1_SetEmulation : trying to set emulation for %s as %s",
	     filename, (Bbinaryemulate ? "FLS" : "KNK"));

	for (j = 0; j < (UINT8) strlen(filename); j++)
		filename[j] = toupper(filename[j]);

	if (!Bbinaryemulate) {
		memcpy(&U32extid,
		       &filename[strlen(filename) - sizeof(U32extid)],
		       sizeof(U32extid));
		if (U32extid != KNK_EXTENSION_ID) {
			flash_errno = FE_INVALID_FILE_FORMAT;
			return FALSE;
		}
	}

	MDBGSTR1("FLS1_SetEmulation : setting emulation from %s",
		 filename);

	strncpy(fls1_config.emulatedflashfilename, filename,
		SYS_MAX_PATHNAME_LEN);
	rfile = fopen(filename, "rb");
	if (!rfile) {
		MDBGSTR1("FLS1_SetEmulation : cannot open %s", filename);
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	if (!Bbinaryemulate) {
		Bsuccess =
		    misc_readheader(rfile, &file_type,
				    &U32storedstartaddr);

		if (!Bsuccess || file_type != FILE_TYPE_FULL
		    || U32storedstartaddr != FLASH_MCU_START_ADDRESS) {
			fclose(rfile);
			flash_errno = FE_INVALID_FILEHEADER;
			return FALSE;
		}
	}

	U32len = misc_getfilesize(rfile);

	if (!Bbinaryemulate) {
		U32len -= sizeof(tFILE_HEADER);
	}

	fls1_config.U32flashendaddr = FLASH_MCU_START_ADDRESS + U32len;
	HDBGSTR1("FLS1_SetEmulation : end of flash is set to "
		 UINT32_FORMAT, fls1_config.U32flashendaddr);
	fclose(rfile);

	fls1_config.Bemulate = TRUE;
	fls1_config.Bbinaryfileemulate = Bbinaryemulate;

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_SetLPTPort(UINT8 U8lptport, UINT16 * U16baseport)
{
	if (U8lptport == 0)
		U8lptport = 1;

	*U16baseport = misc_getLPTport(U8lptport);

	if (*U16baseport == 0) {
		flash_errno = FE_INVALID_LPT_PORT;
		return FALSE;
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
static BOOL FLS1_SetCOMPort(UINT8 U8comport, UINT32 U32comdelay)
{
	if (U8comport < 1 || U8comport > 4)
		return FALSE;

	return MBUS_OpenCOMPort(U8comport, U32comdelay);
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_Terminate(void)
{
	misc_finishredirectionoferror();

#ifdef MBUS
	MBUS_CloseCOMPort();
#endif

#ifdef GENERAL_PORT
	PortDestroy();
#endif
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetForcedFlashSize(UINT8 U8forcedflashsize)
{
	fls1_config.U8forcedflashsize = U8forcedflashsize;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetSkipErase(BOOL Bskiperase)
{
	fls1_config.Bskiperase = Bskiperase;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetDefaultTimes(tFLS1_DELAYBLOCK * pdelayblock)
{
	FLE5_SetDefaultTimes(pdelayblock);
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_GetDefaultTimes(tFLS1_DELAYBLOCK * pdelayblock)
{
	FLE5_GetDefaultTimes(pdelayblock);
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_GeneralSetIdleState(UINT16 U16baseport, UINT32 U32mode)
{
	FLE5_SetIdleState(U16baseport, U32mode);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetIdleState(UINT32 U32mode)
{
	FLE5_SetIdleState(fls1_config.U16baseport, U32mode);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

UINT8 FLS1_GeneralGetSDA(UINT16 U16baseport)
{
	return FLE5_GetSDA(U16baseport);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

UINT8 FLS1_GetSDA(void)
{
	return FLE5_GetSDA(fls1_config.U16baseport);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralWaitForEdge(UINT16 U16baseport, BOOL Bishigh,
			     UINT32 U32retrycount)
{
	return FLE5_WaitForEdge(U16baseport, Bishigh, U32retrycount);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_WaitForEdge(BOOL Bishigh, UINT32 U32retrycount)
{
	return FLE5_WaitForEdge(fls1_config.U16baseport, Bishigh,
				U32retrycount);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralReceiveFromPhone(UINT16 U16baseport, UINT8 * pAU8buffer,
				  UINT32 U32size, UINT32 U32cmd,
				  tFLS1_DELAYBLOCK * pdelayblock)
{
	return FLE5_ReceiveFromPhone(U16baseport, pAU8buffer, U32size,
				     U32cmd, pdelayblock);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ReceiveFromPhone(UINT8 * pAU8buffer, UINT32 U32size,
			   UINT32 U32cmd, tFLS1_DELAYBLOCK * pdelayblock)
{
	return FLE5_ReceiveFromPhone(fls1_config.U16baseport, pAU8buffer,
				     U32size, U32cmd, pdelayblock);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralSendToPhone(UINT16 U16baseport, UINT8 * pAU8buffer,
			     UINT32 U32size, UINT32 U32cmd,
			     tFLS1_DELAYBLOCK * pdelayblock)
{
	return FLE5_SendToPhone(U16baseport, pAU8buffer, U32size, U32cmd,
				pdelayblock);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_SendToPhone(UINT8 * pAU8buffer, UINT32 U32size, UINT32 U32cmd,
		      tFLS1_DELAYBLOCK * pdelayblock)
{
	return FLE5_SendToPhone(fls1_config.U16baseport, pAU8buffer,
				U32size, U32cmd, pdelayblock);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralInitialiseBootstrapDCT3(UINT16 U16baseport,
					 UINT8 * pAU8buffer,
					 UINT16 U16size,
					 UINT32 U32retrycount)
{
	return FLE5_InitialiseBootstrapDCT3(U16baseport, pAU8buffer,
					    U16size, U32retrycount);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_InitialiseBootstrapDCT3(UINT8 * pAU8buffer, UINT16 U16size,
				  UINT32 U32retrycount)
{
	return FLE5_InitialiseBootstrapDCT3(fls1_config.U16baseport,
					    pAU8buffer, U16size,
					    U32retrycount);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_GeneralInitialiseBootstrapDCT4(UINT16 U16baseport,
					 UINT8 * pAU8buffer,
					 UINT16 U16size,
					 UINT32 * pU32mcuresponse)
{
	return FLE5_InitialiseBootstrapDCT4(U16baseport, pAU8buffer,
					    U16size, pU32mcuresponse);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_InitialiseBootstrapDCT4(UINT8 * pAU8buffer, UINT16 U16size,
				  UINT32 * pU32mcuresponse)
{
	return FLE5_InitialiseBootstrapDCT4(fls1_config.U16baseport,
					    pAU8buffer, U16size,
					    pU32mcuresponse);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_MBUSSecurityCommand(const tMBUSCMD * mbuscmd, UINT8 * pU8reply,
			      UINT32 U32replybuffersize,
			      UINT32 * pU32replysize)
{
	return MBUS_SecurityCommand(mbuscmd, pU8reply, U32replybuffersize,
				    pU32replysize);
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetLogCallout(pLogCallout logcallout)
{
	misc_setlogcallout(logcallout);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

eDEVICE_STATE FLS1_GetDeviceState(void)
{
	return device_state;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

eFLASH_ERROR FLS1_GetErrno(void)
{
	return flash_errno;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_GetStatus(tFLASH_STATUS * pstatus)
{
	memcpy(pstatus, &fls1_config.status, sizeof(*pstatus));
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

/*
void FLS1_SetLoopDelay(void)
{
  misc_setdelayfunc(misc_loopdelay);
  FLE5_SetLoop();
}
*/


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

/*
void FLS1_SetRdtscDelay(void)
{
  misc_setdelayfunc(misc_rdtscdelay);
  FLE5_SetTick();
}
*/


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/
/*
void FLS1_SetBaseDelay(UINT32 U32delay)
{
  misc_setbasedelay(U32delay);
}
*/


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetFlashIDList(char *name)
{
	memset(fls1_config.flashid_list, 0,
	       sizeof(fls1_config.flashid_list));
	strncpy(fls1_config.flashid_list, name,
		sizeof(fls1_config.flashid_list) - 1);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetAlgoCodeExtension(char *ext)
{
	memset(fls1_config.dct3_extension_id, 0,
	       sizeof(fls1_config.dct3_extension_id));
	strncpy(fls1_config.dct3_extension_id, ext,
		sizeof(fls1_config.dct3_extension_id) - 1);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetAliasIDList(char *name)
{
	memset(fls1_config.aliasid_list, 0,
	       sizeof(fls1_config.aliasid_list));
	strncpy(fls1_config.aliasid_list, name,
		sizeof(fls1_config.aliasid_list) - 1);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetModelsList(char *name)
{
	memset(fls1_config.models_list, 0,
	       sizeof(fls1_config.models_list));
	strncpy(fls1_config.models_list, name,
		sizeof(fls1_config.models_list) - 1);
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetAlgoCodeList(char *name)
{
	memset(fls1_config.algocode_list, 0,
	       sizeof(fls1_config.algocode_list));
	strncpy(fls1_config.algocode_list, name,
		sizeof(fls1_config.algocode_list) - 1);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ReadFullFlash(char *flashfilename, BOOL Bbinary)
{
	return FLS1_GeneralReadLarge(FLASH_MCU_START_ADDRESS,
				     fls1_config.U32flashendaddr,
				     flashfilename, Bbinary, FALSE, FALSE,
				     NULL);
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetGaugeInit(pGaugeInit gaugeinit)
{
	GaugeInit = gaugeinit;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetGaugeDone(pGaugeDone gaugedone)
{
	GaugeDone = gaugedone;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetGaugeUpdate(pGaugeUpdate gaugeupdate)
{
	GaugeUpdate = gaugeupdate;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

void FLS1_SetSafeFlashing(BOOL Bsafeflash)
{
	fls1_config.Bsafeflashing = Bsafeflash;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_ResetSettings(eMBUS_RESET_TYPE reset_type)
{
	if (reset_type < MBUS_RESET_TYPE_NONE)
		reset_type = MBUS_RESET_TYPE_NONE;

	if (reset_type > MBUS_RESET_TYPE_ALL)
		reset_type = MBUS_RESET_TYPE_ALL;

	return MBUS_ResetSettings(reset_type);
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_ResetPhone(void)
{
	return MBUS_ResetPhone();
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_SetSimlock(UINT8 U8lockidx, char *val, BOOL Bopen)
{
	return MBUS_SetSimlock(U8lockidx, val, Bopen);
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ConvertToKNK(char *filename, UINT32 U32startaddr)
{
	char *ptr;
	char newfilename[SYS_MAX_PATHNAME_LEN];
	FILE *rfile, *sfile;
	UINT8 AU8buffer[4096];
	int numread;
	eFILE_TYPE file_type;
	UINT32 U32oldaddr;
	UINT16 i;

	strcpy(newfilename, filename);

	for (i = 0; i < (UINT16) strlen(newfilename); i++) {
		newfilename[i] = toupper(newfilename[i]);
	}

	ptr = strstr(newfilename, FLASH_FLS_EXTENSION);
	if (ptr) {
		*ptr = 0;
	}
	strcat(newfilename, FLASH_KNK_EXTENSION);

	rfile = fopen(filename, "rb");
	if (!rfile) {
		LDBGSTR1("FLS1_ConvertToKNK : cannot open source file %s",
			 filename);
		flash_errno = FE_FILE_NOT_FOUND;
		return FALSE;
	}

	if (misc_readheader(rfile, &file_type, &U32oldaddr)) {
		LDBGSTR1("FLS1_ConvertToKNK : %s is already a KNK file",
			 filename);
		fclose(rfile);
		return TRUE;
	}
	fseek(rfile, 0, SEEK_SET);

	misc_checkfilename(newfilename);
	sfile = fopen(newfilename, "wb");
	if (!sfile) {
		LDBGSTR1("FLS1_ConvertToKNK : cannot create new file %s",
			 newfilename);
		fclose(rfile);
		flash_errno = FE_CANT_CREATE_FILE;
		return FALSE;
	}

	if (!misc_writeheader(sfile, FILE_TYPE_PARTIAL, U32startaddr)) {
		fclose(rfile);
		fclose(sfile);
		flash_errno = FE_CANT_WRITE_HEADER;
		return FALSE;
	}

	do {
		numread = fread(AU8buffer, 1, sizeof(AU8buffer), rfile);
		fwrite(AU8buffer, 1, numread, sfile);
	} while (numread > 0);

	fclose(rfile);
	fclose(sfile);

	LDBGSTR2("FLS1_ConvertToKNK : '%s' converted to '%s'", filename,
		 newfilename);

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_SetContrast(UINT8 U8val)
{
	return MBUS_SetContrast(U8val);
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static void FLS1_encodeFCHK(UINT8 * U8bytes, UINT8 * U8fchk)
{
	UINT8 j, i;
	UINT8 U8highbit, U8highbitprev;

	for (j = 0; j < 4; j++) {
		U8bytes[j] = ~U8bytes[j];
	}

	for (i = 0; i < 0x20; i++) {
		U8highbit = U8bytes[0] >> 7;
		U8bytes[0] <<= 1;

		U8highbitprev = U8bytes[1] >> 7;
		U8bytes[1] <<= 1;
		U8bytes[1] |= U8highbit;
		U8highbit = U8highbitprev;

		U8highbitprev = U8bytes[2] >> 7;
		U8bytes[2] <<= 1;
		U8bytes[2] |= U8highbit;
		U8highbit = U8highbitprev;

		U8highbitprev = U8bytes[3] >> 7;
		U8bytes[3] <<= 1;
		U8bytes[3] |= U8highbit;
		U8highbit = U8highbitprev;

		U8highbitprev = U8fchk[0] >> 7;
		U8fchk[0] <<= 1;
		U8fchk[0] |= U8highbit;
		U8highbit = U8highbitprev;

		U8highbitprev = U8fchk[1] >> 7;
		U8fchk[1] <<= 1;
		U8fchk[1] |= U8highbit;
		U8highbit = U8highbitprev;

		U8highbitprev = U8fchk[2] >> 7;
		U8fchk[2] <<= 1;
		U8fchk[2] |= U8highbit;
		U8highbit = U8highbitprev;

		U8highbitprev = U8fchk[3] >> 7;
		U8fchk[3] <<= 1;
		U8fchk[3] |= U8highbit;

		if (U8highbitprev) {
			U8fchk[0] ^= 0x95;
			U8fchk[1] ^= 0x7E;
			U8fchk[2] ^= 0x49;
			U8fchk[3] ^= 0x20;
		}
	}
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_CalculateFCHK(UINT32 * pU32fchk)
{
	UINT32 U32pos;
	UINT16 i, U16numit;
	UINT8 U8bytes[sizeof(UINT32)];
	UINT8 U8fchk[sizeof(UINT32)];
	UINT8 AU8buffer[4096];
	UINT32 U32AU8bufferidx = sizeof(AU8buffer);
	UINT32 U32toread, U32readlimit = sizeof(AU8buffer);

	*pU32fchk = 0;

	U8fchk[0] = 0x2d;
	U8fchk[1] = 0x7b;
	U8fchk[2] = 0x19;
	U8fchk[3] = 0x5a;

	if (GaugeInit) {
		GaugeInit(FLASH_MCU_START_ADDRESS + FLS1_FCHK_START_OFFSET,
			  fls1_config.U32endofppm + 1);
	}

	LDBGSTR2("FLS1_CalculateFCHK : calculating flash checksum from "
		 UINT32_FORMAT " until " UINT32_FORMAT,
		 FLASH_MCU_START_ADDRESS, fls1_config.U32endofppm);

	U16numit =
	    (UINT16) ((fls1_config.U32endofppm -
		       FLASH_MCU_START_ADDRESS) >> 6) - 1;
	LDBGSTR1("FLS1_CalculateFCHK : number of iterations is %d",
		 U16numit);

	for (i = 0, U32pos =
	     FLASH_MCU_START_ADDRESS + FLS1_FCHK_START_OFFSET;
	     i < U16numit; i++) {
		if (GaugeUpdate) {
			GaugeUpdate(U32pos);
		}

		if (U32AU8bufferidx >= U32readlimit) {
			U32toread = fls1_config.U32endofppm - U32pos;
			if (U32toread > sizeof(AU8buffer)) {
				U32toread = sizeof(AU8buffer);
			}

			if (!FLS1_Read(U32pos, U32toread, AU8buffer)) {
				if (GaugeDone) {
					GaugeDone();
				}
				return FALSE;
			}
			U32readlimit = U32toread;
			U32AU8bufferidx = 0;
		}

		U8bytes[3] = AU8buffer[U32AU8bufferidx];
		U8bytes[2] = AU8buffer[U32AU8bufferidx + 1];

		U32pos += FLS1_FCHK_SKIP_COUNT;
		U32AU8bufferidx += FLS1_FCHK_SKIP_COUNT;

		if (U32AU8bufferidx >= U32readlimit) {
			U32toread = fls1_config.U32endofppm - U32pos;
			if (U32toread > sizeof(AU8buffer)) {
				U32toread = sizeof(AU8buffer);
			}

			if (!FLS1_Read(U32pos, U32toread, AU8buffer)) {
				if (GaugeDone) {
					GaugeDone();
				}
				return FALSE;
			}
			U32readlimit = U32toread;
			U32AU8bufferidx = 0;
		}

		U8bytes[1] = AU8buffer[U32AU8bufferidx];
		U8bytes[0] = AU8buffer[U32AU8bufferidx + 1];

		U32pos += FLS1_FCHK_SKIP_COUNT;
		U32AU8bufferidx += FLS1_FCHK_SKIP_COUNT;

		FLS1_encodeFCHK(U8bytes, U8fchk);
	}

	for (i = 0; i < sizeof(U8bytes); i++) {
		U8bytes[i] = 0xff;
	}
	FLS1_encodeFCHK(U8bytes, U8fchk);

	for (i = 0; i < sizeof(U8bytes); i++) {
		U8bytes[i] = 0x0;
	}
	FLS1_encodeFCHK(U8bytes, U8fchk);

	memcpy(pU32fchk, U8fchk, sizeof(*pU32fchk));

	*pU32fchk = misc_swapdword(*pU32fchk);

	HDBGSTR1("FLS1_CalculateFCHK : calculated checksum is "
		 UINT32_FORMAT, *pU32fchk);

	if (GaugeDone) {
		GaugeDone();
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_SetFAID(UINT32 U32fchk)
{
	return MBUS_SetFAID(U32fchk);
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_ParseTeslaHeader(FILE * rfile, UINT32 * pU32start,
				  UINT32 * pU32blocklen,
				  UINT8 * pU8fblockchecksum)
{
	tWINTESLA_HEADER header;
	UINT32 U32numread, U32start, U32len;
	UINT8 U8checksum;

	*pU32blocklen = *pU8fblockchecksum = 0;
	U32numread = fread(&header, 1, sizeof(header), rfile);
	if (U32numread == 0) {
		return TRUE;
	}

	if (U32numread != sizeof(header)
	    || header.U8headerid != TESLA_HEADER_ID) {
		LDBGSTR("FLS1_ParseTeslaHeader : not a tesla file");
		return FALSE;
	}

	U32start = 0;
	memcpy(&U32start, header.U8startaddr, sizeof(header.U8startaddr));
	U32start <<= 8;
	if (*pU32start == 0) {
		*pU32start = misc_swapdword(U32start);
	}

	U32len = 0;
	memcpy(&U32len, header.U8blocklen, sizeof(header.U8blocklen));
	U32len <<= 8;
	*pU32blocklen = misc_swapdword(U32len);

	*pU8fblockchecksum = header.U8fblockchecksum;

	U8checksum =
	    header.U8startaddr[0] + header.U8startaddr[1] +
	    header.U8startaddr[2] + header.U8fblockchecksum +
	    header.U8blocklen[0] + header.U8blocklen[1] +
	    header.U8blocklen[2];
	U8checksum ^= 0xFF;

	if (U8checksum != header.U8headerchecksum) {
		LDBGSTR
		    ("FLS1_ParseTeslaHeader : invalid checksum in header");
		return FALSE;
	}

	return TRUE;
}

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

static BOOL FLS1_ConvertWTFileToFLSKNK(char *filename, char *dest,
				       BOOL Btoconvert, BOOL BtoKNK,
				       UINT32 * pU32start)
{
	char newfilename[SYS_MAX_PATHNAME_LEN];
	FILE *rfile, *sfile;
	static UINT8 AU8blockbuffer[MAX_WINTESLA_BLOCK_SIZE];
	UINT32 U32towrite, U32numread, U32blocklen =
	    1, U32desired, U32total = 0;
	UINT32 i, U32start;
	UINT8 U8fblockchecksum, U8calcedfblockchecksum;
	char id[4];
	BOOL BisPPM;
	BOOL Bdetect = TRUE;
	BOOL Btorename;

	if (!filename) {
		return TRUE;
	}

	if (strlen(filename) == 0) {
		LDBGSTR
		    ("FLS1_ConvertWTFileToFLSKNK : no argument is specified");
		return FALSE;
	}

	rfile = fopen(filename, "rb");
	if (!rfile) {
		LDBGSTR1("FLS1_ConvertWTFileToFLSKNK : cannot open %s",
			 filename);
		return FALSE;
	}

	if (dest && strlen(dest) > 0) {
		sfile = fopen(dest, "a+b");
	} else {
		sfile = fopen(FLASH_TEMP_FLASH_FILE, "wb");
	}

	if (!sfile) {
		LDBGSTR1("FLS1_ConvertWTFileToKNK : cannot create %s",
			 newfilename);
		fclose(rfile);
		return FALSE;
	}

	U32start = 0;

	while (U32blocklen > 0) {
		if (!FLS1_ParseTeslaHeader
		    (rfile, &U32start, &U32blocklen, &U8fblockchecksum)) {
			LDBGSTR1
			    ("FLS1_ConvertWTFileToKNK : %s is not a valid WinTesla flash file",
			     filename);
			fclose(rfile);
			fclose(sfile);
			return FALSE;
		}

		if (U32blocklen > sizeof(AU8blockbuffer)) {
			LDBGSTR1
			    ("FLS1_ConvertWTFileToKNK : block is too big (%d)",
			     U32blocklen);
			fclose(rfile);
			fclose(sfile);
			return FALSE;
		}

		if (U32blocklen > 0) {
			U32numread =
			    fread(AU8blockbuffer, 1, U32blocklen, rfile);
			for (i = 0, U8calcedfblockchecksum = 0;
			     i < U32numread; i++) {
				U8calcedfblockchecksum -=
				    AU8blockbuffer[i];
			}
			if (U8fblockchecksum != U8calcedfblockchecksum) {
				LDBGSTR1
				    ("FLS1_ConvertWTFileToKNK : block checksum error in %s",
				     filename);
				fclose(rfile);
				fclose(sfile);
				return FALSE;
			}

			if (Bdetect) {
				Bdetect = FALSE;
				memcpy(id, AU8blockbuffer, sizeof(id) - 1);
				id[sizeof(id) - 1] = 0;
				BisPPM = strcmp(id, PPM_ID) == 0;
			}

			U32total += U32numread;
			fwrite(AU8blockbuffer, 1, U32numread, sfile);
		}
	}
	fclose(rfile);

	if (!BisPPM) {
		U32desired = (U32total + 0x8000) & 0xFF0000;
		U32towrite = U32desired - U32total;

		memset(AU8blockbuffer, 0xFF, sizeof(AU8blockbuffer));
		while (U32towrite > 0) {
			if (U32towrite > sizeof(AU8blockbuffer)) {
				fwrite(AU8blockbuffer, 1,
				       sizeof(AU8blockbuffer), sfile);
				U32towrite -= sizeof(AU8blockbuffer);
			} else {
				fwrite(AU8blockbuffer, 1, U32towrite,
				       sfile);
				U32towrite = 0;
			}
		}
	}

	fclose(sfile);

	if (*pU32start == 0) {
		*pU32start = U32start;
	} else {
		U32start = *pU32start;
	}

	HDBGSTR1("FLS1_ConvertWTFileToKNK : internal start address is "
		 UINT32_FORMAT, U32start);

	if (dest && strlen(dest) > 0) {
		strcpy(newfilename, dest);
		Btorename = FALSE;
	} else {
		if (!BtoKNK) {
			sprintf(newfilename, "%s " UINT32_FORMAT "%s.%s",
				filename, U32start, (BisPPM ? "PPM" : ""),
				FLASH_FLS_EXTENSION);
		} else {
			sprintf(newfilename, "%s %s.%s", filename,
				(BisPPM ? " PPM" : ""),
				FLASH_FLS_EXTENSION);
		}
		misc_checkfilename(newfilename);
		strcpy(dest, newfilename);

		Btorename = TRUE;
	}

	HDBGSTR1("FLS1_ConvertWTFileToFLSKNK : new file is %s",
		 newfilename);

	if (Btorename) {
		HDBGSTR2("FLS1_ConvertWTFileToFLSKNK : renaming %s to %s",
			 FLASH_TEMP_FLASH_FILE, newfilename);
		if (rename(FLASH_TEMP_FLASH_FILE, newfilename)) {
			HDBGSTR2
			    ("FLS1_ConvertWTFileToFLSKNK : cannot rename %s to %s",
			     FLASH_TEMP_FLASH_FILE, newfilename);
			return FALSE;
		}
	}

	if (Btoconvert && BtoKNK) {
		if (!FLS1_ConvertToKNK(newfilename, U32start)) {
			HDBGSTR
			    ("FLS1_ConvertWTFileToFLSKNK : conversion error");
			return FALSE;
		}
		unlink(newfilename);
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

BOOL FLS1_ConvertWTFilesToFLSKNK(char *mcufile, char *ppmfile, BOOL BtoKNK)
{
	char newfilename[SYS_MAX_PATHNAME_LEN];
	BOOL Btwofiles = ppmfile && strlen(ppmfile) > 0;
	BOOL Btoconvert;
	UINT32 U32start = 0;

	Btoconvert = (Btwofiles ? FALSE : BtoKNK);

	newfilename[0] = 0;
	if (!FLS1_ConvertWTFileToFLSKNK
	    (mcufile, newfilename, !Btwofiles, BtoKNK, &U32start)) {
		return FALSE;
	}

	if (Btwofiles) {
		return FLS1_ConvertWTFileToFLSKNK(ppmfile, newfilename,
						  TRUE, BtoKNK, &U32start);
	}

	return TRUE;
}


/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_ResetSPlock(void)
{
	return MBUS_ResetSPlock();
}
#endif

/********************************************************
*                                                       *
* FUNCTION NAME:                                        *
*                                                       *
* ARGUMENTS:                                            *
*                                                       *
* ARGUMENT NAME:                                        *
*                                                       *
* TYPE:                                                 *
*                                                       *
* I/O:                                                  *
*                                                       *
* DESCRIPTION                                           *
*                                                       *
*                                                       *
* RETURNS:                                              *
*                                                       *
*********************************************************/

#ifdef MBUS
BOOL FLS1_SetIMEI(UINT8 * newIMEI)
{
	return MBUS_SetIMEI(newIMEI);
}
#endif
